@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -255,6 +255,7 @@ mforms::ToolBar *ModelDiagramForm::get_tools_toolbar()
         if (item->get_name() == DEFAULT_TOOL)
           item->set_checked(true);
       }
+      toolbar[t]->reset_references();
     }
   }
   return _tools_toolbar;
@@ -314,7 +315,7 @@ void ModelDiagramForm::revalidate_menu()
     _menu->set_item_enabled("wb.edit.selectSimilar", has_selection_);
     _menu->set_item_enabled("wb.edit.selectConnected", get_selection().count() == 1);
 
-    _menu->set_item_checked("wb.edit.toggleGridAlign", get_diagram_options().get_int("AlignToGrid", 0) != 0);
+    _menu->set_item_checked("wb.edit.toggleGridAlign", _owner->get_grt_manager()->get_app_option_int("AlignToGrid", 0) != 0);
     _menu->set_item_checked("wb.edit.toggleGrid", get_diagram_options().get_int("ShowGrid", 1) != 0);
     _menu->set_item_checked("wb.edit.togglePageGrid", get_diagram_options().get_int("ShowPageGrid", 1) != 0);
     _menu->set_item_checked("wb.edit.toggleFKHighlight", get_diagram_options().get_int("ShowFKHighlight", 0) != 0);
@@ -356,7 +357,7 @@ void ModelDiagramForm::update_options_toolbar()
     toolbar = get_wb()->get_component_named("physical")->get_tool_options(get_tool());
   if (!toolbar.is_valid())
     return;
-  
+
   for (size_t c= toolbar->items().count(), i= 0; i < c; i++)
   {
     app_ToolbarItemRef titem(toolbar->items()[i]);
@@ -409,6 +410,9 @@ void ModelDiagramForm::update_options_toolbar()
     
     _options_toolbar->add_item(item);
   }
+
+  // don't reset refs here, only do it when closing, since the toolbar object will be used later
+  //toolbar->reset_references();
 }
 
 
@@ -492,7 +496,7 @@ void ModelDiagramForm::attach_canvas_view(mdc::CanvasView *cview)
   _view= cview;
 
   cview->set_tag(_model_diagram.id());
-  cview->set_grid_snapping(_model_diagram->options().get_int("AlignToGrid", 0)!=0);
+  cview->set_grid_snapping(_owner->get_grt_manager()->get_app_option_int("AlignToGrid", 0) != 0);
   cview->get_background_layer()->set_grid_visible(_model_diagram->options().get_int("ShowGrid", 1)!=0);
   cview->get_background_layer()->set_paper_visible(_model_diagram->options().get_int("ShowPageGrid", 1)!=0);
 
@@ -763,6 +767,9 @@ mdc::CanvasItem *ModelDiagramForm::get_leaf_item_at(const Point &pos)
 
 void ModelDiagramForm::handle_mouse_button(mdc::MouseButton button, bool press, int x, int y, mdc::EventState state)
 {
+  if (_features)
+    _features->tooltip_cancel();
+
   stop_editing();
   
   Point pos(_view->window_to_canvas(x, y));
@@ -866,7 +873,7 @@ void ModelDiagramForm::handle_mouse_move(int x, int y, mdc::EventState state)
 {
   Point pos(_view->window_to_canvas(x, y));
   
-  _features->mouse_moved(x, y, state);
+  //_features->mouse_moved(x, y, state);
   
   _current_mouse_x= x;
   _current_mouse_y= y;
@@ -881,7 +888,11 @@ void ModelDiagramForm::handle_mouse_move(int x, int y, mdc::EventState state)
 bool ModelDiagramForm::handle_key(const mdc::KeyInfo &key, bool press, mdc::EventState state)
 {
   if (press)
-  {    
+  {
+    // cancel tooltip on keypress
+    if (_features)
+      _features->tooltip_cancel();
+
     for (std::vector<WBShortcut>::const_iterator iter= _shortcuts.begin();
       iter != _shortcuts.end(); ++iter)
     {
@@ -1333,6 +1344,12 @@ std::string ModelDiagramForm::get_diagram_info_text()
   return "";
 }
 
+std::vector<std::string> ModelDiagramForm::get_accepted_drop_types()
+{
+  std::vector<std::string> vec;
+  vec.push_back(WB_DBOBJECT_DRAG_TYPE);
+  return vec;
+}
 
 grt::ListRef<model_Object> ModelDiagramForm::get_selection()
 {
@@ -1642,32 +1659,29 @@ bool ModelDiagramForm::relocate_figures()
   return relocated;
 }
 
-
-std::vector<std::string> ModelDiagramForm::get_accepted_drop_types()
-{
-  std::vector<std::string> types= _owner->get_accepted_drop_types();
-
-  return types;
-}
-
+//--------------------------------------------------------------------------------------------------
 
 bool ModelDiagramForm::accepts_drop(int x, int y, const std::string &type, const std::list<GrtObjectRef> &objects)
 {
   return _owner->accepts_drop(this, x, y, type, objects);
 }
 
+//--------------------------------------------------------------------------------------------------
 
 bool ModelDiagramForm::accepts_drop(int x, int y, const std::string &type, const std::string &text)
 {
   return _owner->accepts_drop(this, x, y, type, text);
 }
 
+//--------------------------------------------------------------------------------------------------
 
 bool ModelDiagramForm::perform_drop(int x, int y, const std::string &type, const std::list<GrtObjectRef> &objects)
 {
   return _owner->perform_drop(this, x, y, type, objects);
 }
 
+//--------------------------------------------------------------------------------------------------
+
 bool ModelDiagramForm::perform_drop(int x, int y, const std::string &type, const std::string &text)
 {
   return _owner->perform_drop(this, x, y, type, text);
