@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -43,6 +43,7 @@
 #include "sqlide/wb_sql_editor_form.h"
 #include "sqlide/wb_sql_editor_snippets.h"
 #include "sqlide/wb_sql_editor_help.h"
+#include "sqlide/wb_sql_editor_result_panel.h"
 #include "sqlide/wb_sql_editor_tree_controller.h"
 #include "grt/common.h"
 
@@ -52,7 +53,7 @@
 #include "objimpl/db.query/db_query_QueryBuffer.h"
 #include "grts/structs.db.query.h"
 
-#include "grtdb/db_object_helpers.h"
+#include "grtdb/db_helpers.h"
 
 #include "mforms/code_editor.h"
 #include "mforms/menubar.h"
@@ -216,12 +217,19 @@ public:
     { 
       ref->grt_manager()->replace_status_text("Executing query...");
       
-      RecordsetsRef rsets(ref->exec_sql_returning_results(sql, true));
+      try
+      {
+        RecordsetsRef rsets(ref->exec_sql_returning_results(sql, true));
 
-      for (std::vector<Recordset::Ref>::const_iterator iter= rsets->begin(); iter != rsets->end(); ++iter)
-        result.insert(grtwrap_recordset(_self, *iter));
+        for (std::vector<Recordset::Ref>::const_iterator iter= rsets->begin(); iter != rsets->end(); ++iter)
+          result.insert(grtwrap_recordset(_self, *iter));
       
-      ref->grt_manager()->replace_status_text("Query finished.");
+        ref->grt_manager()->replace_status_text("Query finished.");
+      }
+      catch (sql::SQLException &exc)
+      {
+        log_error("Execption executing SQL code from GRT interface: %s\n", exc.what());
+      }
     }
     return result;
   }
@@ -251,6 +259,28 @@ public:
     if (ref)
       ref->exec_management_sql(sql, log);
   }
+  
+  virtual db_query_ResultsetRef executeQuery(const std::string &sql, bool log)
+  {
+    boost::shared_ptr<SqlEditorForm> ref(_editor);
+    if (ref)
+    {
+      return ref->exec_main_query(sql, log);
+    }
+    return db_query_ResultsetRef();
+  }
+  
+  virtual void executeCommand(const std::string &sql, bool log, bool background)
+  {
+    boost::shared_ptr<SqlEditorForm> ref(_editor);
+    if (ref)
+    {
+      if (background)
+        ref->exec_sql_retaining_editor_contents(sql, Sql_editor::Ref(), false);
+      else
+        ref->exec_main_sql(sql, log);
+    }
+  }
 
   virtual db_query_EditableResultsetRef createTableEditResultset(const std::string &schema, const std::string &table, const std::string &where, bool showGrid)
   {
@@ -387,14 +417,14 @@ static void call_export(wb::WBContextSQLIDE *sqlide)
   {
     int editor = form->active_sql_editor_index();
     if (editor >= 0)
-      form->show_export_recordset(editor, form->active_recordset(editor));
+      form->active_result_panel(editor)->show_export_recordset();
   }
 }
 
 static bool validate_export(wb::WBContextSQLIDE *sqlide)
 {
   SqlEditorForm *form = sqlide->get_active_sql_editor();
-  return form && form->active_sql_editor_index() >= 0 && form->active_recordset(form->active_sql_editor_index());
+  return form && form->active_sql_editor_index() >= 0 && form->active_result_panel(form->active_sql_editor_index());
 }
 
 static void call_run_file(wb::WBContextSQLIDE *sqlide)
@@ -766,10 +796,11 @@ void WBContextSQLIDE::init()
   cmdui->add_builtin_command("query.commit", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::commit));
   cmdui->add_builtin_command("query.rollback", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::rollback));
   cmdui->add_builtin_command("query.autocommit", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::toggle_autocommit));
-  cmdui->add_builtin_command("query.showFieldInfo", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::toggle_show_field_info));
+  cmdui->add_builtin_command("query.gatherFieldInfo", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::toggle_collect_field_info));
+  cmdui->add_builtin_command("query.gatherPSInfo", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::toggle_collect_ps_statement_events));
 
   cmdui->add_builtin_command("query.new_schema", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, &SqlEditorForm::toolbar_command, "query.new_schema"));
-  cmdui->add_builtin_command("query.show_schema_inspector", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, &SqlEditorForm::toolbar_command, "query.show_schema_inspector"));
+  cmdui->add_builtin_command("query.show_inspector", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, &SqlEditorForm::toolbar_command, "query.show_inspector"));
   cmdui->add_builtin_command("query.new_table", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, &SqlEditorForm::toolbar_command, "query.new_table")); 
   cmdui->add_builtin_command("query.new_view", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, &SqlEditorForm::toolbar_command, "query.new_view"));
   cmdui->add_builtin_command("query.new_routine", boost::bind(&WBContextSQLIDE::call_in_editor_str, this, &SqlEditorForm::toolbar_command, "query.new_routine"));
@@ -800,6 +831,7 @@ void WBContextSQLIDE::init()
   cmdui->add_builtin_command("query.explain_current_statement",
                                                boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::explain_current_statement));
 
+  cmdui->add_builtin_command("query.jump_to_placeholder", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::jump_to_placeholder));
   cmdui->add_builtin_command("list-members", boost::bind(&WBContextSQLIDE::call_in_editor, this, &SqlEditorForm::list_members),
                              boost::bind(validate_list_members, this));
 }
@@ -898,9 +930,10 @@ static void *connect_editor(SqlEditorForm::Ref editor, boost::shared_ptr<sql::Tu
   return new std::string();
 }
 
-static bool cancel_connect_editor()
+static bool cancel_connect_editor(SqlEditorForm::Ref editor)
 {
   log_debug3("Cancelling connection...\n");
+  editor->cancel_connect();
   return true;
 }
 
@@ -916,7 +949,7 @@ SqlEditorForm::Ref WBContextSQLIDE::create_connected_editor(const db_mgmt_Connec
                                               strfmt(_("An SQL editor instance for '%s' is opening and should be available in a "
                                                        "moment.\n\nPlease stand by..."), conn->name().c_str()),
                                               boost::bind(connect_editor, editor, tunnel),
-                                              boost::bind(cancel_connect_editor),
+                                              boost::bind(cancel_connect_editor, editor),
                                               result_ptr))
     throw grt::user_cancelled("canceled");
   if (!result_ptr)
@@ -958,10 +991,7 @@ SqlEditorForm::Ref WBContextSQLIDE::create_connected_editor(const db_mgmt_Connec
       std::time_t time = std::time(0);
       parameter_values.gset("lastConnected", (long)time);
 
-      GrtVersionRef version = CatalogHelper::parse_version(object.get_grt(), details["dbmsProductVersion"]);
-      version->owner(object);
-      version->name(grt::StringRef(details["dbmsProductName"]));
-      object->serverVersion(version);
+      object->serverVersion(editor->rdbms_version());
     }
 
     _wbui->get_wb()->get_root()->sqlEditors().insert(object);
