@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -200,27 +200,17 @@ std::string ModelFile::create_document_dir(const std::string &dir, const std::st
 }
 
 
-struct AutoLock
-{
-  GStaticRecMutex *mx;
-  AutoLock(GStaticRecMutex &mutex) : mx(&mutex) { g_static_rec_mutex_lock(mx); }
-  ~AutoLock() { g_static_rec_mutex_unlock(mx); }
-};
-
 
 ModelFile::ModelFile(const std::string &tmpdir)
 : _temp_dir_lock(0), _dirty(false)
 {
   _temp_dir= tmpdir;
-
-  g_static_rec_mutex_init(&_mutex);
 }
 
 
 ModelFile::~ModelFile()
 {
   cleanup();
-  g_static_rec_mutex_free(&_mutex);
 }
 
 
@@ -244,7 +234,7 @@ void ModelFile::open(const std::string &path, GRTManager *grtm)
   bool file_is_zip;
   bool file_is_autosave = false;
 
-  AutoLock lock(_mutex);
+  RecMutexLock lock(_mutex);
 
   if (base::is_directory(path) && path[path.size()-1] == 'd')
   {
@@ -356,7 +346,7 @@ void ModelFile::open(const std::string &path, GRTManager *grtm)
 
       if (g_file_test((auto_save_dir+"/"+MAIN_DOCUMENT_AUTOSAVE_NAME).c_str(), G_FILE_TEST_EXISTS))
       {
-        g_warning("Commiting autosaved document XML file: %s",
+        g_warning("Committing autosaved document XML file: %s",
                   (auto_save_dir+"/"+MAIN_DOCUMENT_AUTOSAVE_NAME).c_str());
         g_remove((auto_save_dir+"/"+MAIN_DOCUMENT_NAME).c_str());
         int rc = g_rename((auto_save_dir+"/"+MAIN_DOCUMENT_AUTOSAVE_NAME).c_str(), (auto_save_dir+"/"+MAIN_DOCUMENT_NAME).c_str());
@@ -460,7 +450,7 @@ std::string ModelFile::read_comment(const std::string &path)
 
 void ModelFile::create(GRTManager *grtm)
 {
-  AutoLock lock(_mutex);
+  RecMutexLock lock(_mutex);
 
   _content_dir= create_document_dir(_temp_dir, "newmodel.mwb");
   add_db_file(grtm, _content_dir);
@@ -479,7 +469,7 @@ std::string ModelFile::get_path_for(const std::string &file)
 // reading
 workbench_DocumentRef ModelFile::retrieve_document(grt::GRT *grt)
 {
-  AutoLock lock(_mutex);
+  RecMutexLock lock(_mutex);
 
   xmlDocPtr xmldoc= grt->load_xml(get_path_for(MAIN_DOCUMENT_NAME));
 
@@ -820,7 +810,7 @@ workbench_DocumentRef ModelFile::unserialize_document(grt::GRT *grt, xmlDocPtr x
  */
 bool ModelFile::save_to(const std::string &path, const std::string &comment)
 {
-  AutoLock lock(_mutex);
+  RecMutexLock lock(_mutex);
 #ifdef _WIN32
   const int read_write= _S_IWRITE|_S_IREAD;
 #else
@@ -900,7 +890,7 @@ bool ModelFile::save_to(const std::string &path, const std::string &comment)
 
 void ModelFile::cleanup()
 {
-  AutoLock lock(_mutex);
+  RecMutexLock lock(_mutex);
 
   delete _temp_dir_lock;
   _temp_dir_lock = 0;
@@ -1016,7 +1006,7 @@ std::string ModelFile::add_note_file(const std::string &path)
 
 bool ModelFile::has_file(const std::string &name)
 {
-  AutoLock lock(_mutex);
+  RecMutexLock lock(_mutex);
 
   return g_file_test(get_path_for(name).c_str(), G_FILE_TEST_EXISTS) != 0;
 }
