@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,9 +20,10 @@
 
 #include "stdafx.h"
 
+#include <grtpp_module_python.h>
 #include <grtpp_module_cpp.h>
 #include <grtpp_module_lua.h>
-#include <grtpp_module_python.h>
+
 #include <python_context.h>
 
 #include "grt/grt_manager.h"
@@ -46,15 +47,13 @@ static GThread *main_thread= 0;
 
 std::map<grt::GRT*,GRTManager*> GRTManager::_instances;
 
-
-GStaticMutex _instance_mutex = G_STATIC_MUTEX_INIT;
-
+static base::Mutex _instance_mutex;
 
 static void init_all()
 {
   if (!main_thread)
    {
-    if (!g_thread_supported()) g_thread_init(NULL);
+    base::threading_init();
     main_thread= g_thread_self();
     if (!g_thread_supported())
       throw std::runtime_error("Could not initialize Glib thread support");
@@ -77,10 +76,6 @@ GRTManager::GRTManager(bool threaded, bool verbose)
 
   init_all();
 
-  _idle_mutex= g_mutex_new();
-  _timer_mutex= g_mutex_new();
-  _disp_map_mutex= g_mutex_new();
-
   _grt= new GRT();
   
   _grt->set_verbose(verbose);
@@ -92,7 +87,7 @@ GRTManager::GRTManager(bool threaded, bool verbose)
   // add self to the mgr instances table asap, because the other objects
   // may need to call get_instance_for()
   {
-    GStaticMutexLock _lock(_instance_mutex);
+    base::MutexLock _lock(_instance_mutex);
     _instances[_grt]= this;
   }
   
@@ -103,14 +98,20 @@ GRTManager::GRTManager(bool threaded, bool verbose)
   _plugin_manager= _grt->get_native_module<PluginManagerImpl>();
 
   _messages_list= new MessageListStorage(this);
-}
 
+  PythonContext::set_run_once_when_idle(boost::bind(&GRTManager::run_once_when_idle_, this, _1));
+}
 
 bool GRTManager::try_soft_lock_globals_tree()
 {
   // returns true if lock count was 0 and then lock it
-  if (g_atomic_int_exchange_and_add(&_globals_tree_soft_lock_count, 1) == 0)
+#if GLIB_CHECK_VERSION(2,32,0)
+  if (g_atomic_int_add(&_globals_tree_soft_lock_count, 1) == 0)
     return true;
+#else
+  if (g_atomic_int_exchange_and_add(&_globals_tree_soft_lock_count, 1) == 0)
+      return true;
+#endif
   // lock failed, decrement it back
   g_atomic_int_add(&_globals_tree_soft_lock_count, -1);
   return false;
@@ -135,7 +136,7 @@ bool GRTManager::is_globals_tree_locked()
 
 GRTManager *GRTManager::get_instance_for(GRT *grt)
 {
-  GStaticMutexLock _lock(_instance_mutex);
+  base::MutexLock lock(_instance_mutex);
   std::map<GRT*,GRTManager*>::iterator iter= _instances.find(grt);
   if (iter != _instances.end())
     return iter->second;
@@ -211,7 +212,7 @@ bool GRTManager::in_main_thread()
 GRTManager::~GRTManager()
 {
   {
-    GStaticMutexLock _lock(_instance_mutex);
+    base::MutexLock _lock(_instance_mutex);
     _instances.erase(_grt);
   }  
 
@@ -229,9 +230,6 @@ GRTManager::~GRTManager()
   for (std::list<Timer*>::iterator iter= _timers.begin(); iter != _timers.end(); ++iter)
     delete *iter;
 
-  g_mutex_free(_disp_map_mutex);
-  g_mutex_free(_timer_mutex);
-  g_mutex_free(_idle_mutex);
 }
 
 
@@ -292,14 +290,14 @@ void GRTManager::execute_grt_task(const std::string &title,
 
 void GRTManager::add_dispatcher(GRTDispatcher::Ref disp)
 {
-  GMutexLock disp_map_mutex(_disp_map_mutex);
+  MutexLock disp_map_mutex(_disp_map_mutex);
   _disp_map[disp];
 }
 
 
 void GRTManager::remove_dispatcher(GRTDispatcher *disp)
 {
-  GMutexLock disp_map_mutex(_disp_map_mutex);
+  MutexLock disp_map_mutex(_disp_map_mutex);
   for (DispMap::iterator i= _disp_map.begin(), end= _disp_map.end(); i != end; ++i)
   {
     if (i->first.get() == disp)
@@ -389,7 +387,7 @@ bool GRTManager::initialize_shell(const std::string &shell_type)
 bool GRTManager::cancel_idle_tasks()
 {
 //   { TODO
-//     GMutexLock disp_map_mutex(_disp_map_mutex);
+//     MutexLock disp_map_mutex(_disp_map_mutex);
 //     for (DispMap::iterator i = _disp_map.begin(), i_end = _disp_map.end(); i != i_end; ++i)
 //       i->first->cancel_all_tasks();
 //   }
@@ -398,7 +396,7 @@ bool GRTManager::cancel_idle_tasks()
     return false;
 
   block_idle_tasks(); // TODO: use idle mutex.
-  GMutexLock lock(_idle_mutex);
+  MutexLock lock(_idle_mutex);
   _current_idle_signal = 0;
   _idle_signals[0].disconnect_all_slots();
   _idle_signals[1].disconnect_all_slots();
@@ -417,7 +415,7 @@ void GRTManager::perform_idle_tasks()
     DispMap copy;
     
     {
-      GMutexLock disp_map_mutex(_disp_map_mutex);
+      MutexLock disp_map_mutex(_disp_map_mutex);
       copy= _disp_map;
     }
     
@@ -439,7 +437,7 @@ void GRTManager::perform_idle_tasks()
 
       int signal_to_emit = 0;
       {
-        GMutexLock lock(_idle_mutex);
+        MutexLock lock(_idle_mutex);
         signal_to_emit = _current_idle_signal;
         _current_idle_signal = _current_idle_signal ? 0 : 1;
       }
@@ -466,17 +464,19 @@ boost::signals2::connection GRTManager::run_once_when_idle(const boost::function
   if (!slot)
     throw std::invalid_argument("Adding null slot for idle");
 
-  GMutexLock lock(_idle_mutex);
+  MutexLock lock(_idle_mutex);
   return _idle_signals[_current_idle_signal].connect(slot);
 }
 
 
-void GRTManager::run_once_when_idle(base::trackable *owner, const boost::function<void ()> &slot)
+boost::signals2::connection GRTManager::run_once_when_idle(base::trackable *owner, const boost::function<void ()> &slot)
 {
   if (!slot)
     throw std::invalid_argument("Adding null slot for idle");
-  GMutexLock lock(_idle_mutex);
-  owner->track_connection(_idle_signals[_current_idle_signal].connect(slot));
+  MutexLock lock(_idle_mutex);
+  boost::signals2::connection tmp(_idle_signals[_current_idle_signal].connect(slot));
+  owner->track_connection(tmp);
+  return tmp;
 }
 
 
@@ -534,22 +534,24 @@ GRTManager::Timer *GRTManager::run_every(const boost::function<bool ()> &slot, d
 
   double delay= timer->delay_for_next_trigger(now);
 
-  g_mutex_lock(_timer_mutex);
-  // insert it in order of delay for next trigger
-  bool inserted= false;
-  for (std::list<Timer*>::iterator iter= _timers.begin(); iter != _timers.end(); ++iter)
   {
-    if ((*iter)->delay_for_next_trigger(now) > delay)
-    {
-      _timers.insert(iter, timer);
-      inserted= true;
-      break;
-    }
-  }
-  if (!inserted)
-    _timers.push_back(timer);
-  g_mutex_unlock(_timer_mutex);
+    base::MutexLock lock(_timer_mutex);
 
+    // insert it in order of delay for next trigger
+    bool inserted= false;
+    for (std::list<Timer*>::iterator iter= _timers.begin(); iter != _timers.end(); ++iter)
+    {
+      if ((*iter)->delay_for_next_trigger(now) > delay)
+      {
+        _timers.insert(iter, timer);
+        inserted= true;
+        break;
+      }
+    }
+    if (!inserted)
+      _timers.push_back(timer);
+
+  }
   _timeout_request();
 
   return timer;
@@ -558,7 +560,7 @@ GRTManager::Timer *GRTManager::run_every(const boost::function<bool ()> &slot, d
 
 void GRTManager::cancel_timer(GRTManager::Timer *timer)
 {
-  g_mutex_lock(_timer_mutex);
+  base::MutexLock lock(_timer_mutex);
   std::list<Timer*>::iterator it= std::find(_timers.begin(), _timers.end(), timer);
   if (it != _timers.end())
   {
@@ -569,7 +571,6 @@ void GRTManager::cancel_timer(GRTManager::Timer *timer)
     _cancelled_timers.insert(timer);
   // if the timer is not in the timers list, then it may be getting executed,
   // so add it to a list of timers so it doesn't get readded to the timers list
-  g_mutex_unlock(_timer_mutex);
 }
 
 
@@ -581,22 +582,24 @@ void GRTManager::flush_timers()
   std::list<Timer*> triggered;
 
   // first get a list of timers that trigger now
-  g_mutex_lock(_timer_mutex);
+
   std::list<Timer*>::iterator next, iter= _timers.begin();
-  while (iter != _timers.end())
   {
-    next= iter;
-    ++next;
+    base::MutexLock lock(_timer_mutex);
+    while (iter != _timers.end())
+    {
+      next= iter;
+      ++next;
 
-    if ((*iter)->delay_for_next_trigger(now) > 0.00001)
-      break;
-    
-    triggered.push_back(*iter);
-    _timers.erase(iter);
+      if ((*iter)->delay_for_next_trigger(now) > 0.00001)
+        break;
 
-    iter= next;
+      triggered.push_back(*iter);
+      _timers.erase(iter);
+
+      iter= next;
+    }
   }
-  g_mutex_unlock(_timer_mutex);
 
   // after this point it's impossible for the timer to be cancelled
   // because it is not in the timers list anymore
@@ -611,7 +614,7 @@ void GRTManager::flush_timers()
     {
       double delay= (*iter)->delay_for_next_trigger(now);
 
-      g_mutex_lock(_timer_mutex);
+      base::MutexLock lock(_timer_mutex);
 
       if (_cancelled_timers.find(*iter) == _cancelled_timers.end())
       {
@@ -632,18 +635,15 @@ void GRTManager::flush_timers()
       else
         delete *iter;
 
-      g_mutex_unlock(_timer_mutex);
     }
     else
     {
-      g_mutex_lock(_timer_mutex);
+      base::MutexLock lock(_timer_mutex);
       delete *iter;
-      g_mutex_unlock(_timer_mutex);
     }
   }
-  g_mutex_lock(_timer_mutex);
+  base::MutexLock lock(_timer_mutex);
   _cancelled_timers.clear();
-  g_mutex_unlock(_timer_mutex);
 }
 
 
@@ -651,7 +651,7 @@ double GRTManager::delay_for_next_timeout()
 {
   double delay= -1;
 
-  g_mutex_lock(_timer_mutex);
+  base::MutexLock lock(_timer_mutex);
   if (!_timers.empty())
   {
     GTimeVal now;
@@ -660,7 +660,6 @@ double GRTManager::delay_for_next_timeout()
     if (delay < 0)
       delay= 0.0;
   }
-  g_mutex_unlock(_timer_mutex);
 
   return delay;
 }
