@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -78,7 +78,7 @@ DbDriverParam::ParamType DbDriverParam::decode_param_type(std::string type_name)
   else if (0 == type_name.compare("text"))
     result= ptText;
   else
-    g_warning("Unkown DB driver parameter type '%s'", type_name.c_str());
+    g_warning("Unknown DB driver parameter type '%s'", type_name.c_str());
 
   return result;
 }
