@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -89,6 +89,7 @@ grt::DoubleRef WBRecordsetResultset::floatFieldValue(long column)
   if (column >= 0 && column < recordset->get_column_count() &&
       recordset->get_field(cursor, column, value))
     return grt::DoubleRef(value);
+  throw std::invalid_argument(base::strfmt("invalid column %li for resultset", column).c_str());
   return grt::DoubleRef(0.0);
 }
 
@@ -101,6 +102,7 @@ grt::DoubleRef WBRecordsetResultset::floatFieldValueByName(const std::string &co
     if (recordset->get_field(cursor, column_by_name[column], value))
       return grt::DoubleRef(value);
   }
+  throw std::invalid_argument(base::strfmt("invalid column %s for resultset", column.c_str()).c_str());
   return grt::DoubleRef(0.0);
 }
 
@@ -140,6 +142,7 @@ grt::IntegerRef WBRecordsetResultset::intFieldValue(long column)
   if (column >= 0 && column < recordset->get_column_count() &&
       recordset->get_field(bec::NodeId(cursor), (int)column, value))
     return grt::IntegerRef((grt::IntegerRef::storage_type)value);
+  throw std::invalid_argument(base::strfmt("invalid column %li for resultset", column).c_str());
   return grt::IntegerRef(0);
 }
 
@@ -152,13 +155,14 @@ grt::IntegerRef WBRecordsetResultset::intFieldValueByName(const std::string &col
     if (recordset->get_field(bec::NodeId(cursor), column_by_name[column], value))
       return grt::IntegerRef((grt::IntegerRef::storage_type)value);
   }
+  throw std::invalid_argument(base::strfmt("invalid column %s for resultset", column.c_str()).c_str());
   return grt::IntegerRef(0);
 }
 
 
 grt::IntegerRef WBRecordsetResultset::nextRow()
 {
-  if ((int) cursor < recordset->count() - 1)
+  if ((int) cursor < recordset->row_count() - 1)
   {
     ++cursor;
     return grt::IntegerRef(1);
@@ -190,6 +194,7 @@ grt::StringRef WBRecordsetResultset::stringFieldValue(long column)
   if (column >= 0 && column < recordset->get_column_count() &&
       recordset->get_field_repr_no_truncate(bec::NodeId(cursor), (int)column, value))
     return grt::StringRef(value);
+  throw std::invalid_argument(base::strfmt("invalid column %li for resultset", column).c_str());
   return grt::StringRef(); // NULL
 }
 
@@ -202,6 +207,7 @@ grt::StringRef WBRecordsetResultset::stringFieldValueByName(const std::string &c
     if (recordset->get_field_repr_no_truncate(bec::NodeId(cursor), column_by_name[column], value))
       return grt::StringRef(value);
   }
+  throw std::invalid_argument(base::strfmt("invalid column %s for resultset", column.c_str()).c_str());
   return grt::StringRef(); // NULL
 }
 
@@ -321,6 +327,7 @@ public:
   {
     if (column >= 0 && column < (long)column_by_name.size())
       return grt::DoubleRef(recordset->getDouble(column+1));
+    throw std::invalid_argument(base::strfmt("invalid column %li for resultset", column).c_str());
     return grt::DoubleRef(0.0);
   }
 
@@ -331,6 +338,7 @@ public:
     {
       return grt::DoubleRef(recordset->getDouble(column_by_name[column]));
     }
+    throw std::invalid_argument(base::strfmt("invalid column %s for resultset", column.c_str()).c_str());
     return grt::DoubleRef(0.0);
   }
 
@@ -355,8 +363,9 @@ public:
 
   virtual grt::IntegerRef intFieldValue(long column)
   {
-    if (column > 0 && column < (long)column_by_name.size())
+    if (column >= 0 && column < (long)column_by_name.size())
       return grt::IntegerRef(recordset->getInt(column+1));
+    throw std::invalid_argument(base::strfmt("invalid column %li for resultset", column).c_str());
     return grt::IntegerRef(0);
   }
 
@@ -367,6 +376,7 @@ public:
     {
       return grt::IntegerRef(recordset->getInt(column_by_name[column]));
     }
+    throw std::invalid_argument(base::strfmt("invalid column %s for resultset", column.c_str()).c_str());
     return grt::IntegerRef(0);
   }
 
@@ -392,6 +402,7 @@ public:
   {
     if (column >= 0 && column < (long)column_by_name.size())
       return grt::StringRef(recordset->getString(column+1));
+    throw std::invalid_argument(base::strfmt("invalid column %li for resultset", column).c_str());
     return grt::StringRef(); // NULL
   }
 
@@ -402,6 +413,7 @@ public:
     {
       return grt::StringRef(recordset->getString(column_by_name[column]));
     }
+    throw std::invalid_argument(base::strfmt("invalid column %s for resultset", column.c_str()).c_str());
     return grt::StringRef(); // NULL
   }
   
