@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,6 +23,7 @@
 #include "recordset_cdbc_storage.h"
 #include "recordset_be.h"
 #include "sqlide_generics_private.h"
+#include "sqlide_generics.h"
 #include "grtsqlparser/sql_facade.h"
 #include "base/string_utilities.h"
 #include <sqlite/query.hpp>
@@ -58,6 +59,12 @@ sql::Dbc_connection_handler::ConnectionRef Recordset_cdbc_storage::dbms_conn_ref
   return _dbms_conn->ref;
 }
 
+sql::Dbc_connection_handler::ConnectionRef Recordset_cdbc_storage::aux_dbms_conn_ref()
+{
+  if (!_aux_dbms_conn || !_aux_dbms_conn->ref.get_ptr())
+  throw std::runtime_error("No connection to DBMS");
+  return _aux_dbms_conn->ref;
+}
 
 class FetchVar : public boost::static_visitor<sqlite::variant_t>
 {
@@ -249,21 +256,24 @@ void Recordset_cdbc_storage::do_unserialize(Recordset *recordset, sqlite::connec
 
   if (_gather_field_info)
   {
-    std::stringstream ss;
     for (ColumnId i = 0; i < editable_col_count; ++i)
     {
-      ss << "Field " << i+1 << ": " << rs_meta->getColumnLabel(i+1) << "\n";
-      ss << "Catalog: " << rs_meta->getCatalogName(i+1) << "\n";
-      ss << "Schema: " << rs_meta->getSchemaName(i+1) << "\n";
-      ss << "Table: " << rs_meta->getTableName(i+1) << "\n";
-      ss << "Type: " << rs_meta->getColumnTypeName(i+1) << "\n";
+      FieldInfo info;
+      info.catalog = rs_meta->getCatalogName(i+1);
+      info.schema = rs_meta->getSchemaName(i+1);
+      info.table = rs_meta->getTableName(i+1);
+      info.field = rs_meta->getColumnLabel(i+1);
+      info.type = rs_meta->getColumnTypeName(i+1);
 //      ss << "Collation:" << "\n";
-      ss << "Display size: " << rs_meta->getColumnDisplaySize(i+1) << "\n";
-      ss << "Precision: " << rs_meta->getPrecision(i+1) << "\n";
-      ss << "Scale: " << rs_meta->getScale(i+1) << "\n";
-      ss << "\n";
+      info.display_size = rs_meta->getColumnDisplaySize(i+1);
+      info.precision = rs_meta->getPrecision(i+1);
+      info.scale = rs_meta->getScale(i+1);
+      _field_info.push_back(info);
     }
-    _field_info = ss.str();
+      
+    // Turns the flag off to prevent unnecesary
+    // reprocessing to get the field info.
+    _gather_field_info = false;
   }
 
   column_types.reserve(editable_col_count);
@@ -285,33 +295,42 @@ void Recordset_cdbc_storage::do_unserialize(Recordset *recordset, sqlite::connec
   for (ColumnId n= column_names.size(); editable_col_count > n; ++n)
     column_names.push_back(rs_meta->getColumnLabel(n+1));
 
-  // pkey columns
+  // determine pkey or unique identifier columns
   ColumnId rowid_col_count= 0;
-  if (!_table_name.empty() && !_readonly) // we need PK info only for EDIT statements and table_name member is filled only for EDIT statements
+  if (!_table_name.empty()) // we need PK info only for editable statements and table_name member is filled only for those
   {
-    sql::DatabaseMetaData *conn_meta(dbms_conn->getMetaData());
+    // a connection other than the user connection must be used for fetching metadata, otherwise we change the state of the connection
+    sql::Dbc_connection_handler::ConnectionRef aux_dbms_conn_ref= this->aux_dbms_conn_ref();
+
+    sql::DatabaseMetaData *conn_meta(aux_dbms_conn_ref->getMetaData());
     std::auto_ptr<sql::ResultSet> rs(conn_meta->getBestRowIdentifier("", _schema_name, _table_name, 0, 0));
     rowid_col_count= rs->rowsCount();
-    if (!rowid_col_count)
-      throw std::runtime_error(strfmt(_("%s: table data is not editable because there is no primary key defined for the table"), full_table_name().c_str()));
-    _pkey_columns.reserve(rowid_col_count);
-    column_names.reserve(editable_col_count + rowid_col_count);
-    column_types.reserve(editable_col_count + rowid_col_count);
-    real_column_types.reserve(editable_col_count + rowid_col_count);
-    while (rs->next())
+    if (rowid_col_count > 0)
     {
-      Recordset::Column_names::const_iterator i=
-        std::find(column_names.begin(), column_names.end(), rs->getString("COLUMN_NAME"));
-      if (i != column_names.end())
+      _pkey_columns.reserve(rowid_col_count);
+      column_names.reserve(editable_col_count + rowid_col_count);
+      column_types.reserve(editable_col_count + rowid_col_count);
+      real_column_types.reserve(editable_col_count + rowid_col_count);
+      while (rs->next())
       {
-        ColumnId col= std::distance((Recordset::Column_names::const_iterator)column_names.begin(), i);
-        column_names.push_back(column_names[col]);
-        column_types.push_back(column_types[col]);
-        real_column_types.push_back(real_column_types[col]);
-        _pkey_columns.push_back(col); // copy original value of pk field(s)
+        Recordset::Column_names::const_iterator i=
+          std::find(column_names.begin(), column_names.end(), rs->getString("COLUMN_NAME"));
+        if (i != column_names.end())
+        {
+          ColumnId col= std::distance((Recordset::Column_names::const_iterator)column_names.begin(), i);
+          column_names.push_back(column_names[col]);
+          column_types.push_back(column_types[col]);
+          real_column_types.push_back(real_column_types[col]);
+          _pkey_columns.push_back(col); // copy original value of pk field(s)
+        }
+        else
+          rowid_col_count--;
       }
-      else
-        rowid_col_count--;
+    }
+    else
+    {
+      _readonly = true;
+      _readonly_reason = "The table has no unique row identifier (primary key or a NOT NULL unique index)";
     }
   }
 
@@ -325,13 +344,14 @@ void Recordset_cdbc_storage::do_unserialize(Recordset *recordset, sqlite::connec
 
   // data
   {
-    sqlide::Sqlite_transaction_guarder transaction_guarder(data_swap_db);
+    sqlide::Sqlite_transaction_guarder transaction_guarder(data_swap_db, false);
 
     create_data_swap_tables(data_swap_db, column_names, column_types);
 
     ColumnId col_count= editable_col_count + rowid_col_count;
     FetchVar fetch_var(rs.get());
     Var_vector row_values(col_count);
+
     std::list<boost::shared_ptr<sqlite::command> > insert_commands= prepare_data_swap_record_add_statement(data_swap_db, column_names);
 
     while (rs->next())
