@@ -303,7 +303,12 @@ WBTable::ItemList::iterator WBTable::sync_next_column(ItemList::iterator iter,
     return iter;
   }
 
-  if (flags & wbfig::ColumnPK)
+  if ((flags & (wbfig::ColumnPK|wbfig::ColumnFK)) == (wbfig::ColumnPK|wbfig::ColumnFK))
+    iter= sync_next(_column_box, _columns, iter, id,
+                    mdc::ImageManager::get_instance()->get_image("db.Column.pkfk.11x11.png"), text,
+                    boost::bind(&WBTable::create_column_item, this, _1, _2),
+                    boost::bind(&WBTable::update_column_item, this, _1, flags));
+  else if (flags & wbfig::ColumnPK)
     iter= sync_next(_column_box, _columns, iter, id, 
                     mdc::ImageManager::get_instance()->get_image("db.Column.pk.11x11.png"), text, 
                     boost::bind(&WBTable::create_column_item, this, _1, _2),
