@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -44,17 +44,18 @@ namespace wb {
     * The SidebarEntry class is a lean wrapper for an icon/label combination which can be used
     * to trigger an action. It needs a container to be useful (here the class SidebarEntry).
     */
-  class SidebarEntry
+  class SidebarEntry : public mforms::Accessible
   {
   private:
     std::string _name;
     std::string _title;
     cairo_surface_t* _icon;
     mforms::TaskEntryType _type;
+    base::Rect _bounds;
     bool _enabled;
   public:
     SidebarEntry(const std::string& name, const std::string& title, const std::string& icon, mforms::TaskEntryType type);
-    ~SidebarEntry();
+    virtual ~SidebarEntry();
     
     void set_title(const std::string &title);
     void set_icon(const std::string &icon);
@@ -68,12 +69,18 @@ namespace wb {
     mforms::TaskEntryType type() { return _type; };
     bool enabled() const { return _enabled; }
 
+    // ------ Accesibility Methods -----
+    virtual std::string get_acc_name() { return _title; }
+    virtual mforms::Accessible::Role get_acc_role() { return mforms::Accessible::OutlineItem; }
+    virtual base::Rect get_acc_bounds() { return _bounds;}
   };
 
   class SidebarSection : public mforms::DrawBox
   {
   private:
-    struct Button {
+    struct Button : public mforms::Accessible 
+    {
+      std::string _name;
       cairo_surface_t *icon;
       cairo_surface_t *alt_icon;
       int x, y;
@@ -82,18 +89,26 @@ namespace wb {
       bool hot;
       bool down;
       bool state;
+      base::Rect _bounds;
 
-      Button(const std::string &icon, const std::string &alt_icon);
-      ~Button();
+      Button(const std::string &name, const std::string &icon, const std::string &alt_icon);
+      virtual ~Button();
 
       void draw(cairo_t *cr);
       void move(int x, int y);
 
       bool check_hit(int x, int y);
+
+      // ------ Accesibility Methods -----
+      virtual std::string get_acc_name() { return _name; }
+      virtual mforms::Accessible::Role get_acc_role() { return mforms::Accessible::PushButton;}
+      virtual base::Rect get_acc_bounds() { return base::Rect(x, y, bounds_width, bounds_height);}
+
     };
 
     std::string _name;
     std::string _title;
+    std::vector<Button*> _enabled_buttons;
     std::vector<SidebarEntry*> _entries;
     int _layout_width;
     int _layout_height;
@@ -120,7 +135,6 @@ namespace wb {
     void create_context_for_layout();
     void layout(cairo_t* cr);
     SidebarEntry* entry_from_point(double x, double y);
-
   public:
     SidebarSection(SimpleSidebar* owner, const std::string &name, const std::string& title, mforms::TaskSectionFlags flags);
     ~SidebarSection();
@@ -142,18 +156,26 @@ namespace wb {
     SidebarEntry* selected() { return _selected_entry; };
     bool expanded() { return _expanded; };
     void toggle_expand();
+    
+    void update_mode_button(bool active);
 
     std::string name() { return _name; }
     std::string title() { return _title; };
 
     virtual void repaint(cairo_t* cr, int areax, int areay, int areaw, int areah);
-    virtual void mouse_leave();
-    virtual void mouse_move(int x, int y);
-    virtual void mouse_down(int button, int x, int y);
-    virtual void mouse_click(int button, int x, int y);
-    virtual void mouse_up(int button, int x, int y);
+    virtual bool mouse_leave();
+    virtual bool mouse_move(mforms::MouseButton button, int x, int y);
+    virtual bool mouse_down(mforms::MouseButton button, int x, int y);
+    virtual bool mouse_click(mforms::MouseButton button, int x, int y);
+    virtual bool mouse_up(mforms::MouseButton button, int x, int y);
     virtual void get_layout_size(int* w, int* h);
 
+    virtual mforms::Accessible::Role get_acc_role() { return mforms::Accessible::OutlineItem;}
+    virtual std::string get_acc_name() { return _title; }
+    virtual int get_acc_child_count();
+    virtual Accessible* get_acc_child(int index);
+    virtual mforms::Accessible* hit_test(int x, int y);
+
     void clear_selection();
 
     boost::signals2::signal<void (SidebarSection*)>* expanded_changed() { return &_expanded_changed; }
@@ -200,13 +222,16 @@ namespace wb {
     virtual std::string selected_entry();
     virtual void clear_selection();
 
+    virtual mforms::Accessible::Role get_acc_role() { return mforms::Accessible::Outline; }
+
+    virtual void update_mode_buttons(bool active);
+
   private:
     static bool __init;
     static bool init_factory_method();
   };
 
-  
-  class AdvancedSidebar : public SimpleSidebar, public Observer
+  class AdvancedSidebar : public SimpleSidebar, public base::Observer
   {
   private:
     typedef boost::signals2::signal<void (const std::string&)> SearchBoxChangedSignal;
