@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,8 +18,8 @@
  */
 
 #include "stdafx.h"
-#include "base/string_utilities.h"
 #include "grt_python_debugger.h"
+#include "base/string_utilities.h"
 #include "grt_shell_window.h"
 #include "grt_code_editor.h"
 
@@ -275,7 +275,7 @@ PythonDebugger::PythonDebugger(GRTShellWindow *shell, mforms::TabView *tabview)
 
 void PythonDebugger::init_pdb()
 {
-  grt::WillEnterPython lock;
+  WillEnterPython lock;
   
   grt::PythonContext *pyc = grt::PythonContext::get();
   
@@ -347,7 +347,7 @@ void PythonDebugger::line_gutter_clicked(int margin, int line, mforms::ModifierK
 {  
   if (margin == 1 || margin == 0) // click line numbers or on the markers
   {
-    grt::WillEnterPython lock;
+    WillEnterPython lock;
  
     if (toggle_breakpoint(editor->get_path().c_str(), line+1)) // line numbers from editor are 0 based
       editor->get_editor()->show_markup(LineMarkupBreakpoint, line);
@@ -361,7 +361,7 @@ void PythonDebugger::editor_text_changed(int line, int linesAdded, GRTCodeEditor
 {
   if (linesAdded != 0)
   {
-    grt::WillEnterPython lock;
+    WillEnterPython lock;
     
     std::string path = editor->get_path();
     
@@ -382,7 +382,7 @@ void PythonDebugger::edit_breakpoint(mforms::TreeNodeRef node, int column, std::
   int row = _breakpoint_list->row_for_node(node);
   if (column == 2 && row >= 0) // edit bp condition
   {
-    grt::WillEnterPython lock;
+    WillEnterPython lock;
     grt::AutoPyObject r(PyObject_CallMethod(_pdb, (char*)"wdb_set_bp_condition", (char*)"(is)", row, value.c_str()), 
                         false);
     if (!r)
@@ -404,7 +404,7 @@ void PythonDebugger::edit_breakpoint(mforms::TreeNodeRef node, int column, std::
 
 void PythonDebugger::refresh_file(const std::string &file)
 {
-  grt::WillEnterPython lock;
+  WillEnterPython lock;
   grt::AutoPyObject r(PyObject_CallMethod(_pdb, (char*)"wdb_reload_module_for_file", (char*)"(s)", file.c_str()), 
                       false);
 }
@@ -415,7 +415,7 @@ void PythonDebugger::run(GRTCodeEditor *editor, bool stepping)
   if (editor->is_dirty() && !ensure_code_saved())
     return;
 
-  grt::WillEnterPython lock;
+  WillEnterPython lock;
 
 //  debug_print(base::strfmt("Running script %s...\n", _shell->get_path().c_str()));
 
@@ -738,7 +738,7 @@ void PythonDebugger::stack_selected()
     show_frame = -1*(_stack_list->row_for_node(node)+1);
   }
   
-  grt::WillEnterPython lock;
+  WillEnterPython lock;
   
   grt::AutoPyObject r(PyObject_CallMethod(_pdb, (char*)"wdb_refresh_variables", (char*)"(i)", 
                                           show_frame),
@@ -753,7 +753,7 @@ void PythonDebugger::stack_selected()
 
 bool PythonDebugger::toggle_breakpoint(const char *file, int line)
 {
-  grt::WillEnterPython lock;
+  WillEnterPython lock;
   
   grt::AutoPyObject r(PyObject_CallMethod(_pdb, (char*)"wdb_toggle_breakpoint", (char*)"(si)", 
                                           file, line),
