@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -76,6 +76,7 @@ class DbSqlEditorView : public Gtk::VBox, public FormViewBase
   protected:
     virtual void plugin_tab_added(PluginEditorBase *plugin);
 
+
   private:
     void polish();
 
@@ -88,7 +89,7 @@ class DbSqlEditorView : public Gtk::VBox, public FormViewBase
     void update_resultsets(int editor_index, Recordset::Ref rset, bool added);
     void update_resultsets_from_main();
     std::deque<sigc::slot<void> >  _update_resultset_slots; // Slot will have editor index, resultset attached
-    GMutex                        *_update_resultset_slots_lock;
+    base::Mutex                        _update_resultset_slots_lock;
 
     int  add_editor_tab(int active_sql_editor_index);
     void editor_page_switched(GtkNotebookPage *page, guint index);
@@ -107,7 +108,6 @@ class DbSqlEditorView : public Gtk::VBox, public FormViewBase
 
     void set_maximized_editor_mode(bool flag);
 
-    std::map<Gtk::Widget*,mforms::Menu*> _menu_list;
     SqlEditorForm::Ref    _be;
     Gtk::HPaned           _top_pane;
     Gtk::HPaned           _top_right_pane;
@@ -116,9 +116,13 @@ class DbSqlEditorView : public Gtk::VBox, public FormViewBase
     Gtk::Widget          *_side_palette;
 
     sigc::connection      _polish_conn;
+    sigc::connection      _sig_restore_sidebar;
+    sigc::connection      _sig_close_editor;
+    sigc::connection      _sig_close_plugin;
+    sigc::connection      _sig_close_app;
 
     NotebookDockingPoint  _dock_delegate;
-    mforms::DockingPoint  _dpoint;
+    mforms::DockingPoint  *_dpoint;
 
     Glib::Dispatcher      _dispatch_rset_update;
     sigc::connection      _dispatch_rset_update_conn;
