@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,11 +25,13 @@
 #include "base/util_functions.h"
 #include "base/wb_iterators.h"
 #include "base/log.h"
+#include "mforms/../gtk/lf_native.h"
 #include "mforms/../gtk/lf_view.h"
 #include "mforms/../gtk/lf_toolbar.h"
 #include "objimpl/ui/mforms_ObjectReference_impl.h"
 #include <glib.h>
 #include "grt/common.h"
+#include "sqlide/wb_sql_editor_result_panel.h"
 #include "sqlide_form.h"
 
 DEFAULT_LOG_DOMAIN("UI")
@@ -47,19 +49,19 @@ QueryView::QueryView(const int current_index, DbSqlEditorView* owner)
           , _apply_btn(_("Apply"))
           , _cancel_btn(_("Revert"))
           , _dock_delegate(&_rs_tabs, RESULT_DOCKING_POINT)
-          , _dpoint(&_dock_delegate, false)
           , _query_collapsed(false)
           , _updating_results(false)
 {
+  _dpoint = mforms::manage(new mforms::DockingPoint(&_dock_delegate, false));
   SqlEditorForm::Ref             be = owner->be();
   Sql_editor::Ref            editor = be->sql_editor(current_index);
-  Gtk::Widget&        editor_widget = *mforms::gtk::ViewImpl::get_widget_for_view(editor->get_editor_control());
+  Gtk::Widget&        editor_widget = *mforms::widget_for_view(editor->get_editor_control());
 
   _editor = editor;
 
   {
     db_query_QueryEditorRef qeditor(db_query_QueryEditorRef::cast_from(_editor->grtobj()));
-    qeditor->resultDockingPoint(mforms_to_grt(qeditor->get_grt(), &_dpoint, "DockingPoint"));
+    qeditor->resultDockingPoint(mforms_to_grt(qeditor->get_grt(), _dpoint, "DockingPoint"));
   }
 
   _dock_delegate.notebook_changed_signal.connect(sigc::mem_fun(this, &QueryView::update_lower_notebook_visibility));
@@ -71,7 +73,7 @@ QueryView::QueryView(const int current_index, DbSqlEditorView* owner)
   _rs_tabs.set_tab_pos(Gtk::POS_BOTTOM);
 
   boost::shared_ptr<mforms::ToolBar> toolbar(be->sql_editor_toolbar(be->sql_editor_index(editor)));
-  Gtk::Widget* w = mforms::gtk::widget_for_toolbar(toolbar.get());
+  Gtk::Widget* w = mforms::widget_for_toolbar(toolbar.get());
   if (w)
   {
     w->show();
@@ -118,12 +120,12 @@ QueryView::QueryView(const int current_index, DbSqlEditorView* owner)
 //------------------------------------------------------------------------------
 QueryView::~QueryView()
 {
-  for (std::map<RecordsetView*,mforms::Menu*>::iterator it = _menu_list.begin(); it != _menu_list.end(); ++it)
-  {
-    if (it->second)
-      delete it->second;
-  }
-  _menu_list.clear();
+  //We need to remove all pending gtk::load.
+  if (!_gtk_load.empty())
+    _gtk_load.disconnect();
+
+  if (_dpoint)
+    _dpoint->release();
 }
 
 
@@ -211,8 +213,8 @@ void QueryView::update_resultsets()
 
   SqlEditorForm::Ref             be                  = _owner->be();
   const int                      n_recordsets_in_be  = be->recordset_count(editor_index);
-  std::vector<Recordset::Ref>    recordsets_from_be;
-  std::map<Recordset::Ref, int>  recordset_indexes;
+  std::vector<SqlEditorResult::Ref>  recordsets_from_be;
+  std::map<SqlEditorResult::Ref, int>  recordset_indexes;
 
   _updating_results = true;
 
@@ -223,25 +225,29 @@ void QueryView::update_resultsets()
   {
     for (int i = n_recordsets_in_be - 1; i >= 0; --i)
     {
-      recordsets_from_be.push_back(be->recordset(editor_index, i));
+      recordsets_from_be.push_back(be->result_panel(be->recordset(editor_index, i)));
       recordset_indexes[recordsets_from_be.back()] = i;
     }
   }
 
-  Recordset::Ref              rs;
   const int                   ntabs = _rs_tabs.get_n_pages();
-  std::vector<Gtk::Widget*>   orphan_views; // list of views which Recordset::ref is not in BE
+  std::vector<SqlEditorResult*>   orphan_views; // list of views which Recordset::ref is not in BE
 
   // Walk notebook pages removing existing Recordset objects from the @recordsets_from_be list
   // Also add view to @orphan_views if Recordset is not in BE, for later remove_page
   for (int i = 0; i < ntabs; ++i)
   {
-    RecordsetView* view = dynamic_cast<RecordsetView*>(_rs_tabs.get_nth_page(i));
+    SqlEditorResult* view = dynamic_cast<SqlEditorResult*>(mforms::view_for_widget(_rs_tabs.get_nth_page(i)));
     if (view)
     {
-      rs = view->model();
       const size_t before_remove = recordsets_from_be.size();
-      base::vector_remove(recordsets_from_be, rs);
+      for (std::vector<SqlEditorResult::Ref>::iterator it = recordsets_from_be.begin();
+          it != recordsets_from_be.end(); ++it)
+        if (it->get() == view)
+        {
+          recordsets_from_be.erase(it);
+          break;
+        }
 
       if (recordsets_from_be.size() == before_remove)
         orphan_views.push_back(view);
@@ -250,35 +256,35 @@ void QueryView::update_resultsets()
 
   // Remove orphan pages
   for (size_t i = 0; i < orphan_views.size(); ++i)
-    _rs_tabs.remove_page(*orphan_views[i]);
+    _rs_tabs.remove_page(*mforms::widget_for_view(orphan_views[i]));
 
   // Create new pages
   const int size = recordsets_from_be.size();
   for (int i = 0; i < size; ++i)
   {
-    const Recordset::Ref&        rs = recordsets_from_be[i];
-
-    std::auto_ptr<ActiveLabel>   lbl(new ActiveLabel(rs->caption(), sigc::bind(sigc::mem_fun(this, &QueryView::close_recordset), rs->key(), true)));
-    std::auto_ptr<RecordsetView> view(RecordsetView::create(rs, 0));
+    SqlEditorResult::Ref        result_panel = recordsets_from_be[i];
 
+    std::auto_ptr<ActiveLabel>   lbl(new ActiveLabel(result_panel->caption(), 
+		sigc::bind(sigc::mem_fun(this, &QueryView::close_recordset), result_panel->recordset()->key(), true)));
+    std::auto_ptr<RecordsetView> view(RecordsetView::create(result_panel->recordset()));
 
     if (lbl.get() && view.get())
     {
       lbl->show();
+      result_panel->dock_result_grid(mforms::manage(mforms::native_from_widget(view.get())));
+
+      mforms::Menu* menu = init_tab_menu(view.get());
+      lbl.get()->set_menu(menu, true);
+      menu->set_handler(sigc::bind(sigc::mem_fun(this, &QueryView::tab_menu_handler), lbl.get(), view.get()));
+      widget_for_view(result_panel.get())->set_data("active_label", lbl.get());
+
+      const int pos_added = _rs_tabs.append_page(*Gtk::manage(widget_for_view(result_panel.get())), *(Gtk::manage(lbl.release())));
+      _rs_tabs.set_tab_reorderable(*widget_for_view(result_panel.get()), true);
+
+      view->refresh();
       view->show();
 
-      RecordsetView* rs_view   = view.get();
-
-      mforms::Menu* menu = init_tab_menu(rs_view);
-      lbl.get()->set_menu(menu);
-      menu->set_handler(sigc::bind(sigc::mem_fun(this, &QueryView::tab_menu_handler), lbl.get(), rs_view));
-      rs_view->set_data("active_label", lbl.get());
-
-      const int pos_added = _rs_tabs.append_page(*Gtk::manage(view.release()), *(Gtk::manage(lbl.release())));
-      _rs_tabs.set_tab_reorderable(*rs_view, true);
-
-      rs_view->refresh();
-      rs_view->show();
+      view.release();
 
       _rs_tabs.set_current_page(pos_added);
     }
@@ -305,7 +311,7 @@ void QueryView::update_lower_notebook_visibility(bool added)
     _polish_conn.disconnect();
 
     if (added && _rs_tabs.get_n_pages() == 1) // first tab was added
-      utils::gtk::load_settings(_owner->grt_manager(), &_top_pane, sigc::bind(sigc::ptr_fun(gtk_paned_set_pos_ratio), &_top_pane, 0.4), false, -max_position);
+      _gtk_load = utils::gtk::load_settings(_owner->grt_manager(), &_top_pane, sigc::bind(sigc::ptr_fun(gtk_paned_set_pos_ratio), &_top_pane, 0.4), false, -max_position);
     else if (_rs_tabs.get_n_pages() == 0) // tabs are gone
       Glib::signal_idle().connect_once(sigc::bind(sigc::ptr_fun(hide_rs_pane), &_top_pane));
     else if (_rs_tabs.get_n_pages() > 0) // more tabs was added or replaced
@@ -322,9 +328,9 @@ void QueryView::recalc_rstab_indices()
   const int size = _rs_tabs.get_n_pages();
   for (int i = 0; i < size; ++i)
   {
-    RecordsetView* view = dynamic_cast<RecordsetView*>(_rs_tabs.get_nth_page(i));
+    SqlEditorResult* view = dynamic_cast<SqlEditorResult*>(mforms::view_for_widget(_rs_tabs.get_nth_page(i)));
     if (view)
-      _owner->be()->recordset_reorder(index(), view->model(), i);
+      _owner->be()->recordset_reorder(index(), view->recordset(), i);
   }
 }
 
@@ -334,13 +340,22 @@ void QueryView::close()
   _owner->close_editor_tab(this);
 }
 
+//------------------------------------------------------------------------------
+RecordsetView* QueryView::recordset_view_for_widget(Gtk::Widget *w)
+{
+  SqlEditorResult *result = dynamic_cast<SqlEditorResult*>(mforms::view_for_widget(w));
+  if (result)
+    return dynamic_cast<RecordsetView*>(mforms::widget_for_view(result->result_grid()));
+  return NULL;
+}
+
 //------------------------------------------------------------------------------
 RecordsetView* QueryView::active_recordset()
 {
   RecordsetView* rs(0);
   const int page = _rs_tabs.get_current_page();
   if (page >= 0)
-    rs = dynamic_cast<RecordsetView*>(_rs_tabs.get_nth_page(page));
+    rs = recordset_view_for_widget(_rs_tabs.get_nth_page(page));
 
   return rs;
 }
@@ -352,7 +367,7 @@ void QueryView::close_recordset(long long key, bool confirm)
 
   for (int i = 0; i < ntabs; ++i)
   {
-    RecordsetView* view  = dynamic_cast<RecordsetView*>(_rs_tabs.get_nth_page(i));
+    RecordsetView* view  = recordset_view_for_widget(_rs_tabs.get_nth_page(i));
     if (view)
     {
       const Recordset::Ref model = view->model();
@@ -372,17 +387,6 @@ void QueryView::close_recordset_by_ptr(RecordsetView* view, const bool confirm)
   {
     if (confirm && view->model()->can_close(true))
     {
-      ActiveLabel* al = (ActiveLabel*)view->get_data("active_label");
-      if (al)
-        al->set_menu(NULL);
-
-      std::map<RecordsetView*,mforms::Menu*>::iterator it;
-      it = _menu_list.find(view);
-      if (it!= _menu_list.end())
-      {
-        delete it->second;
-        _menu_list.erase(it);
-      }
       view->model()->close();
      // _rs_tabs.remove_page(*view);
     }
@@ -504,17 +508,12 @@ int QueryView::process_task_msg(int msgType, const std::string &message, const s
 //------------------------------------------------------------------------------
 mforms::Menu* QueryView::init_tab_menu(RecordsetView* rs)
 {
-  std::map<RecordsetView*,mforms::Menu*>::iterator it;
-  it = _menu_list.find(rs);
-  if (it == _menu_list.end())
   {
     mforms::Menu *m = new mforms::Menu();
     m->add_item("Close Tab", "close tab");
     m->add_item("Close Other Tabs", "close other tabs");
-    _menu_list.insert(std::pair<RecordsetView*,mforms::Menu*>(rs,m));
     return m;
   }
-  return it->second;
 }
 
 //------------------------------------------------------------------------------
@@ -532,7 +531,7 @@ void QueryView::tab_menu_handler(const std::string& action, ActiveLabel* sender,
 
     for (int i = 0; i < size; ++i)
     {
-      RecordsetView* view  = dynamic_cast<RecordsetView*>(_rs_tabs.get_nth_page(i));
+      RecordsetView* view  = recordset_view_for_widget(_rs_tabs.get_nth_page(i));
       if (view && rsview != view)
         to_delete.push_back(view);
     }
