@@ -28,15 +28,24 @@
 #include "common.h"
 #endif
 
+#include <map>
+
 #ifdef _WIN32
   #define DEFAULT_FONT_FAMILY "Tahoma"
+  #define DEFAULT_FONT_SIZE 11
+  
+  #define DEFAULT_SMALL_FONT "Modern"
   #define DEFAULT_MONOSPACE_FONT_FAMILY "Consolas"
   #define DEFAULT_MONOSPACE_FONT_FAMILY_ALT "Lucida Console"
 #elif defined(__APPLE__)
   #define DEFAULT_FONT_FAMILY "Helvetica"
-  #define DEFAULT_MONOSPACE_FONT_FAMILY "Andale Mono"
+  #define DEFAULT_FONT_SIZE 11
+
+  #define DEFAULT_MONOSPACE_FONT_FAMILY "AndaleMono"
 #else
   #define DEFAULT_FONT_FAMILY "Helvetica"
+  #define DEFAULT_FONT_SIZE 11
+
   #define DEFAULT_MONOSPACE_FONT_FAMILY "Bitstream Vera Sans Mono"
 #endif
 
@@ -48,7 +57,8 @@ namespace base {
     ColorSchemeStandardWin7,
     ColorSchemeStandardWin8,
     ColorSchemeStandardWin8Alternate,
-    ColorSchemeCustom,
+    ColorSchemeHighContrast,
+    ColorSchemeCustom = 128,
   };
 
   enum ApplicationColor
@@ -91,6 +101,7 @@ namespace base {
     static std::string get_application_color_as_string(ApplicationColor color, bool foreground);
     static void set_active_scheme(ColorScheme scheme);
     static ColorScheme get_active_scheme();
+    static bool is_high_contrast_scheme();
 
     // Persistence support. Also called when colors were changed in preferences.
     static void load_custom_colors(const std::map<std::string, std::string> &colors);
