@@ -47,18 +47,18 @@ namespace mforms {
 #ifndef SWIG
   struct MFORMS_EXPORT TableImplPtrs
   {
-    bool (__stdcall *create)(Table*);
+    bool (*create)(Table*);
 
-    void (__stdcall *set_row_count)(Table *self, int);
-    void (__stdcall *set_column_count)(Table *self, int);
+    void (*set_row_count)(Table *self, int);
+    void (*set_column_count)(Table *self, int);
 
-    void (__stdcall *add)(Table *self,View*, int, int, int, int, int);
-    void (__stdcall *remove)(Table *self, View*);
+    void (*add)(Table *self,View*, int, int, int, int, int);
+    void (*remove)(Table *self, View*);
 
-    void (__stdcall *set_row_spacing)(Table *self, int);
-    void (__stdcall *set_column_spacing)(Table *self, int);
+    void (*set_row_spacing)(Table *self, int);
+    void (*set_column_spacing)(Table *self, int);
 
-    void (__stdcall *set_homogeneous)(Table *self, bool);
+    void (*set_homogeneous)(Table *self, bool);
   };
 #endif
 #endif
