@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2011, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,7 +32,7 @@ namespace mforms {
   enum ToolBarType
   {
     MainToolBar,
-    SecondaryToolBar, // TODO: linux
+    SecondaryToolBar,
     ToolPickerToolBar, // only toggle and separator items are required to work
     OptionsToolBar,
     PaletteToolBar
@@ -50,28 +50,34 @@ namespace mforms {
     ColorSelectorItem,
     SeparatorItem,
     ExpanderItem,
-    ImageBoxItem
+    ImageBoxItem,
+#ifdef __linux__
+    TitleItem = LabelItem // TODO implement specific TitleItem (bold with #333333 color) for Linux
+#else
+    TitleItem
+#endif
   };
+
 #ifndef SWIG
   struct MFORMS_EXPORT ToolBarImplPtrs
   {
-    bool (__stdcall *create_tool_bar)(ToolBar *item, ToolBarType type);
-    void (__stdcall *insert_item)(ToolBar *toolbar, int index, ToolBarItem *item);
-    void (__stdcall *remove_item)(ToolBar *toolbar, ToolBarItem *item);
+    bool (*create_tool_bar)(ToolBar *item, ToolBarType type);
+    void (*insert_item)(ToolBar *toolbar, int index, ToolBarItem *item);
+    void (*remove_item)(ToolBar *toolbar, ToolBarItem *item);
 
-    bool (__stdcall *create_tool_item)(ToolBarItem *item, ToolBarItemType type);
-    void (__stdcall *set_item_icon)(ToolBarItem *item, const std::string&);
-    void (__stdcall *set_item_alt_icon)(ToolBarItem *item, const std::string&);
-    void (__stdcall *set_item_text)(ToolBarItem *item, const std::string&);
-    std::string (__stdcall *get_item_text)(ToolBarItem *item);
-    void (__stdcall *set_item_enabled)(ToolBarItem *item, bool);
-    bool (__stdcall *get_item_enabled)(ToolBarItem *item);
-    void (__stdcall *set_item_checked)(ToolBarItem *item, bool);
-    bool (__stdcall *get_item_checked)(ToolBarItem *item);
-    void (__stdcall *set_item_tooltip)(ToolBarItem *item, const std::string&);
+    bool (*create_tool_item)(ToolBarItem *item, ToolBarItemType type);
+    void (*set_item_icon)(ToolBarItem *item, const std::string&);
+    void (*set_item_alt_icon)(ToolBarItem *item, const std::string&);
+    void (*set_item_text)(ToolBarItem *item, const std::string&);
+    std::string (*get_item_text)(ToolBarItem *item);
+    void (*set_item_enabled)(ToolBarItem *item, bool);
+    bool (*get_item_enabled)(ToolBarItem *item);
+    void (*set_item_checked)(ToolBarItem *item, bool);
+    bool (*get_item_checked)(ToolBarItem *item);
+    void (*set_item_tooltip)(ToolBarItem *item, const std::string&);
 
     // For selector items only.
-    void (__stdcall *set_selector_items)(ToolBarItem* item, const std::vector<std::string>& values);
+    void (*set_selector_items)(ToolBarItem* item, const std::vector<std::string>& values);
   }; 
 #endif
 
@@ -97,17 +103,20 @@ namespace mforms {
     void set_item_enabled(const std::string &name, bool flag);
     void set_item_checked(const std::string &name, bool flag);
     bool get_item_checked(const std::string &name);
-    
+
     void add_item(ToolBarItem *item);
     void insert_item(int index, ToolBarItem *item);
     void remove_all();
     void remove_item(ToolBarItem *item);
-    
+
+    ToolBarItem *add_separator_item(const std::string &name="");
+
     void validate();
   };
   
   class MFORMS_EXPORT ToolBarItem : public Object
   {
+  friend class ToolBar;
   public:
     ToolBarItem(ToolBarItemType type = ActionItem, const bool expandable = false);
     ToolBarItemType get_type() { return _type; }
