@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,46 +24,47 @@
 namespace mforms {
 namespace stub {
 
-class UtilitiesImpl
+class UtilitiesWrapper
 {
   static boost::function<void (std::string)> open_url_slot;
   static boost::function<mforms::DialogResult (void)> message_callback;
   static std::map<const std::string, std::string> passwords;
 
-  static int __stdcall show_message(const std::string &title, const std::string &text,
+  static int show_message(const std::string &title, const std::string &text,
                           const std::string &ok, const std::string &cancel,
                           const std::string &other);
-  static int __stdcall show_error(const std::string &title, const std::string &text,
+  static int show_error(const std::string &title, const std::string &text,
                         const std::string &ok, const std::string &cancel,
                         const std::string &other);
-  static int __stdcall show_warning(const std::string &title, const std::string &text,
+  static int show_warning(const std::string &title, const std::string &text,
                         const std::string &ok, const std::string &cancel,
                         const std::string &other);
-  static int __stdcall show_message_with_checkbox(const std::string &title, const std::string &text,
+  static int show_message_with_checkbox(const std::string &title, const std::string &text,
                         const std::string &ok, const std::string &cancel,
                         const std::string &other,
                         const std::string &checkbox_text, // empty text = default "Don't show this message again" text
                         bool &remember_checked);
 
-  static void __stdcall show_wait_message(const std::string &title, const std::string &text);
-  static bool __stdcall hide_wait_message();
-  static bool __stdcall run_cancelable_wait_message(const std::string &title, const std::string &text,
+  static void show_wait_message(const std::string &title, const std::string &text);
+  static bool hide_wait_message();
+  static bool run_cancelable_wait_message(const std::string &title, const std::string &text,
                         const boost::function<void ()> &start_task, const boost::function<bool ()> &cancel_task);
-  static void __stdcall stop_cancelable_wait_message();
+  static void stop_cancelable_wait_message();
     
-  static void __stdcall set_clipboard_text(const std::string &text);
-  static std::string __stdcall get_clipboard_text();
+  static void set_clipboard_text(const std::string &text);
+  static std::string get_clipboard_text();
 
-  static std::string __stdcall get_special_folder(mforms::FolderType type);
+  static std::string get_special_folder(mforms::FolderType type);
 
 
-  static void __stdcall open_url(const std::string &url);
-  static void __stdcall add_timeout(float interval, const boost::function<bool ()> &slot);
+  static void open_url(const std::string &url);
+  static mforms::TimeoutHandle add_timeout(float interval, const boost::function<bool ()> &slot);
+  static void cancel_timeout(mforms::TimeoutHandle);
 
-  static void __stdcall store_password(const std::string &service, const std::string &account, const std::string &password);
-  static bool __stdcall find_password(const std::string &service, const std::string &account, std::string &password);
-  static void __stdcall forget_password(const std::string &service, const std::string &account);
-  static void* __stdcall perform_from_main_thread(const boost::function<void* ()>& slot, bool wait);
+  static void store_password(const std::string &service, const std::string &account, const std::string &password);
+  static bool find_password(const std::string &service, const std::string &account, std::string &password);
+  static void forget_password(const std::string &service, const std::string &account);
+  static void* perform_from_main_thread(const boost::function<void* ()>& slot, bool wait);
 
 public:
   static void init();
