@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -28,8 +28,6 @@
 #include "base/log.h"
 #include "base/notifications.h"
 
-DEFAULT_LOG_DOMAIN(DOMAIN_MFORMS_BE);
-
 using namespace mforms;
 
 //--------------------------------------------------------------------------------------------------
@@ -52,6 +50,7 @@ View::~View()
   clear_subviews();
 
   // Let the frontend delete all resources it allocated.
+  // XXX: is this really needed given that we have a FreeDataFunc in mforms::Object?
   if (_view_impl->destroy)
     _view_impl->destroy(this);
 }
@@ -291,9 +290,16 @@ void View::set_size(int width, int height)
 
 //--------------------------------------------------------------------------------------------------
 
-void View::client_to_screen(int& x, int& y)
+std::pair<int, int> View::client_to_screen(int x, int y)
 {
-  (*_view_impl->client_to_screen)(this, x, y);
+  return (*_view_impl->client_to_screen)(this, x, y);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+std::pair<int, int> View::screen_to_client(int x, int y)
+{
+  return (*_view_impl->screen_to_client)(this, x, y);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -312,6 +318,13 @@ bool View::is_shown()
 
 //--------------------------------------------------------------------------------------------------
 
+bool View::is_fully_visible()
+{
+  return (*_view_impl->is_fully_visible)(this);
+}
+
+//--------------------------------------------------------------------------------------------------
+
 void View::set_enabled(bool flag)
 {
   (*_view_impl->set_enabled)(this, flag);
@@ -409,7 +422,9 @@ void View::set_back_image(const std::string &path, Alignment align)
 }
 
 //--------------------------------------------------------------------------------------------------
-
+//Below code is used only for debug purpose.
+//It's using the object::retain_count.
+#ifdef _0
 void View::show_retain_counts(int depth)
 {
   printf("%*s '%s' (%i)\n", depth, "--", get_name().c_str(), retain_count());
@@ -420,7 +435,7 @@ void View::show_retain_counts(int depth)
     iter->first->show_retain_counts(depth+1);
   }
 }
-
+#endif
 //--------------------------------------------------------------------------------------------------
 
 void View::flush_events()
@@ -433,10 +448,37 @@ void View::flush_events()
 
 void View::focus()
 {
-  if (_view_impl->focus)
-    _view_impl->focus(this);
-  else
-    log_warning("mforms::View::focus not implemented");
+  _view_impl->focus(this);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void View::register_drop_formats(DropDelegate *target, const std::vector<std::string> &drop_formats)
+{
+  _view_impl->register_drop_formats(this, target, drop_formats);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+DragOperation View::do_drag_drop(DragDetails details, const std::string &text)
+{
+  return _view_impl->drag_text(this, details, text);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+DragOperation View::do_drag_drop(DragDetails details, void *data, const std::string &format)
+{
+  return _view_impl->drag_data(this, details, data, format);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+bool View::mouse_leave()
+{
+  if (_signal_mouse_leave.num_slots() > 0)
+    return _signal_mouse_leave();
+  return false;
 }
 
 //--------------------------------------------------------------------------------------------------
