@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,175 +17,107 @@
  * 02110-1301  USA
  */
 
-/**
- * Contains the implementation of the wrapper for a .NET form used by the mforms backend.
- */
-
 #include "stdafx.h"
 
 #include "base/drawing.h"
 
-#include "mforms/mforms.h"
+#include "wf_base.h"
 #include "wf_view.h"
 #include "wf_box.h"
 #include "wf_appview.h"
-#include "wf_menubar.h"
 #include "wf_toolbar.h"
 
-#include "ConvUtils.h"
-
 using namespace System::Drawing::Drawing2D;
+using namespace System::Windows::Forms;
 
 using namespace MySQL::Forms;
 using namespace MySQL::Controls;
 
-//--------------------------------------------------------------------------------------------------
-
-bool AppViewImpl::create(mforms::AppView *self, bool horizontal)
-{
-  AppViewImpl^ appview = gcnew AppViewImpl(self);
-
-  if (appview != nullptr)
-  {
-    // The next steps are the same as with a BoxImpl.
-    Box^ native_box = ViewImpl::create<Box>(self, appview);
-    native_box->Horizontal = horizontal;
-    appview->set_resize_mode(AutoResizeMode::ResizeNone);
-
-    return true;
-  }
-  return false;
-}
-
-//--------------------------------------------------------------------------------------------------
-
-/**
- * Called when this app view is about to be docked in a host container. Create the front end
- * tab document if not yet done and return it to the caller.
- */
-TabDocument^ AppViewImpl::GetHost()
-{
-  if (_host == nullptr)
-    setup();
-  return _host;
-}
-
-//--------------------------------------------------------------------------------------------------
-
-MenuStrip^ AppViewImpl::GetMenuBar()
-{
-  mforms::MenuBar* menu = _appView->get_menubar();
-  if (menu == NULL)
-    return nullptr;
-
-  MenuBarImpl^ menubar_impl = (MenuBarImpl^) ViewImpl::FromUnmanaged(menu);
-
-  return menubar_impl->get_control<MenuStrip>();
-}
-
-//--------------------------------------------------------------------------------------------------
-
-ToolStrip^ AppViewImpl::GetToolBar()
-{
-  mforms::ToolBar* toolbar = _appView->get_toolbar();
-  if (toolbar == NULL)
-    return nullptr;
-
-  ToolBarImpl^ toolbar_impl = (ToolBarImpl^) ViewImpl::FromUnmanaged(toolbar);
-
-  return toolbar_impl->get_control<ToolStrip>();
-}
-
-//--------------------------------------------------------------------------------------------------
-
-String^ AppViewImpl::GetTitle()
-{
-  return CppStringToNativeRaw(_appView->get_title());
-}
-
-//--------------------------------------------------------------------------------------------------
-
-bool AppViewImpl::DocumentClosing()
-{
-  Control^ control= get_control<Control>();
-
-  if (control->Tag != nullptr)
-  {
-    mforms::AppView* view= ViewImpl::get_backend_control<mforms::AppView>(control);
-    if (view != NULL)
-      return view->on_close();
-  }
-  return true;
-}
-
-//--------------------------------------------------------------------------------------------------
-
-AppViewImpl::AppViewImpl(mforms::AppView* app)
-  : BoxImpl(app), _appView(app)
-{
-}
-
-//--------------------------------------------------------------------------------------------------
-
-AppViewImpl::~AppViewImpl()
-{
-  delete _host;
-}
-
-//--------------------------------------------------------------------------------------------------
-
-void AppViewImpl::setup()
-{
-  // In order to ease maintenance we create a special document host for our content.
-  _host = gcnew AppViewDockContent(this);
-  Control^ control = get_control<Control>();
-  control->BackColor = Drawing::Color::Transparent;
-  _host->Controls->Add(control);
-  control->Dock = DockStyle::Fill;
-}
-
 //----------------- AppViewDockContent -------------------------------------------------------------
 
-AppViewDockContent::AppViewDockContent(AppViewImpl^ appview)
+AppViewDockContent::AppViewDockContent()
 {
-  _appview = appview;
-  _appview->get_unmanaged_object()->retain();
+  appview = NULL;
 };
 
 //--------------------------------------------------------------------------------------------------
 
 AppViewDockContent::~AppViewDockContent()
 {
-  _appview->get_unmanaged_object()->release();
-  _appview= nullptr;
+  if (appview != NULL)
+    appview->release();
 }
 
 //--------------------------------------------------------------------------------------------------
 
-AppViewImpl^ AppViewDockContent::GetAppView()
+void AppViewDockContent::SetBackend(mforms::AppView *backend)
 {
-  return _appview;
-};
+  appview = backend;
+  appview->retain();
+}
 
 //--------------------------------------------------------------------------------------------------
 
-mforms::AppView* AppViewDockContent::get_unmanaged_object()
+mforms::AppView* AppViewDockContent::GetBackend()
 {
-  return _appview->get_unmanaged_object();
+  return appview;
 }
 
 //--------------------------------------------------------------------------------------------------
 
 String^ AppViewDockContent::GetAppViewIdentifier()
 {
-  return CppStringToNative(_appview->get_unmanaged_object()->identifier());
+  return CppStringToNative(appview->identifier());
 }
 
 //--------------------------------------------------------------------------------------------------
 
 String^ MySQL::Forms::AppViewDockContent::GetContextName()
 {
-  return CppStringToNative(_appview->get_unmanaged_object()->get_form_context_name());
+  return CppStringToNative(appview->get_form_context_name());
+}
+
+//--------------------------------------------------------------------------------------------------
+
+MenuStrip^ AppViewDockContent::GetMenuBar()
+{
+  mforms::MenuBar *menu = appview->get_menubar();
+  if (menu == NULL)
+    return nullptr;
+
+  return AppViewWrapper::GetManagedObject<MenuStrip>(menu);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+ToolStrip^ AppViewDockContent::GetToolBar()
+{
+  mforms::ToolBar *toolbar = appview->get_toolbar();
+  if (toolbar == NULL)
+    return nullptr;
+
+  return AppViewWrapper::GetManagedObject<ToolStrip>(toolbar);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+bool AppViewDockContent::DocumentClosing()
+{
+  return appview->on_close();
+}
+
+//--------------------------------------------------------------------------------------------------
+
+String^ AppViewDockContent::GetTitle()
+{
+  return CppStringToNativeRaw(appview->get_title());
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void AppViewDockContent::SetTitle(String^ title)
+{
+  appview->set_title(NativeToCppStringRaw(title));
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -199,14 +131,18 @@ void AppViewDockContent::UpdateColors()
   else
     BackColor = Conversions::GetApplicationColor(ApplicationColor::AppColorMainBackground, false);
 
-  MenuStrip ^menuStrip = _appview->GetMenuBar();
+  MenuStrip ^menuStrip = GetMenuBar();
   if (menuStrip != nullptr)
   {
     menuStrip->BackColor = Conversions::GetApplicationColor(ApplicationColor::AppColorPanelToolbar, false);
     menuStrip->ForeColor = Conversions::GetApplicationColor(ApplicationColor::AppColorPanelToolbar, true);
+    if (Conversions::UseWin8Drawing())
+      menuStrip->Renderer = gcnew Win8MenuStripRenderer();
+    else
+      menuStrip->Renderer = gcnew TransparentMenuStripRenderer();
   }
 
-  ToolStrip ^toolStrip = _appview->GetToolBar();
+  ToolStrip ^toolStrip = GetToolBar();
   if (toolStrip != nullptr)
   {
     toolStrip->BackColor = Conversions::GetApplicationColor(ApplicationColor::AppColorPanelToolbar, false);
@@ -214,5 +150,52 @@ void AppViewDockContent::UpdateColors()
   }
 }
 
+//----------------- AppViewWrapper -----------------------------------------------------------------
+
+AppViewWrapper::AppViewWrapper(mforms::AppView* app)
+  : BoxWrapper(app), appview(app)
+{
+}
+
 //--------------------------------------------------------------------------------------------------
 
+bool AppViewWrapper::create(mforms::AppView *backend, bool horizontal)
+{
+  AppViewWrapper *wrapper = new AppViewWrapper(backend);
+  wrapper->set_resize_mode(AutoResizeMode::ResizeNone);
+
+  // In order to ease maintenance we create a special document host for our content.
+  // This adds another nesting level, however.
+  LayoutBox ^box = Create<LayoutBox>(backend, wrapper);
+  box->Horizontal = horizontal;
+
+  wrapper->host = gcnew AppViewDockContent();
+  wrapper->host->SetBackend(backend);
+
+  box->BackColor = Drawing::Color::Transparent;
+  wrapper->host->Controls->Add(box);
+  box->Dock = DockStyle::Fill;
+
+  return true;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Called when this app view is about to be docked in a host container. Create the frontend
+ * tab document if not yet done and return it to the caller.
+ */
+AppViewDockContent^ AppViewWrapper::GetHost()
+{
+  return host;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+void AppViewWrapper::init()
+{
+  mforms::ControlFactory *f = mforms::ControlFactory::get_instance();
+  f->_app_view_impl.create = &AppViewWrapper::create;
+}
+
+//--------------------------------------------------------------------------------------------------
