@@ -19,29 +19,21 @@
 
 #pragma once
 
-#include "wf_button.h"
-
 namespace MySQL {
   namespace Forms {
 
-    public ref class CheckBoxImpl : public ButtonImpl
+    public class CheckBoxWrapper : public ButtonWrapper
     {
-      static bool create(mforms::CheckBox *self, bool square);
-      static void click(System::Object ^sender, System::EventArgs ^e);
-      static void set_active(mforms::CheckBox *self, bool flag);
-      static bool get_active(mforms::CheckBox *self);
+    protected:
+      CheckBoxWrapper(mforms::CheckBox *cbox);
 
-      CheckBoxImpl(mforms::CheckBox *cbox);
+      static bool create(mforms::CheckBox *backend, bool square);
+      static void click(System::Object ^sender, System::EventArgs ^e);
+      static void set_active(mforms::CheckBox *backend, bool flag);
+      static bool get_active(mforms::CheckBox *backend);
 
     public:
-      static void init(Manager ^mgr)
-      {
-        mforms::ControlFactory *f= mforms::ControlFactory::get_instance();
-
-        DEF_CALLBACK2(bool, mforms::CheckBox*, bool, mgr, f->_checkbox_impl, CheckBoxImpl, create);
-        DEF_CALLBACK2(void, mforms::CheckBox*, bool, mgr, f->_checkbox_impl, CheckBoxImpl, set_active);
-        DEF_CALLBACK1(bool, mforms::CheckBox*, mgr, f->_checkbox_impl, CheckBoxImpl, get_active);
-      }
+      static void init();
     };
 
   };
