@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014444, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -117,7 +117,7 @@ static MYX_GRT_SHELL_COMMAND_HELP_TEXT help_shell[] =
     "show",
     "Prints the current global object or the object with the given path.",
     "show [path]",
-    "path           The absolut (starting with /) or relative (starting with ./) path"NL
+    "path           The absolute (starting with /) or relative (starting with ./) path"NL
       "               of the object to print.",
     "  cd Migration"NL
       "               Changes the current global object to ./Migration. If the current"NL
