@@ -33,11 +33,7 @@
 #undef max
 #endif
 
-/**
- * @file  grtpp_undo_manager.cpp
- * @brief 
- */
-
+DEFAULT_LOG_DOMAIN("Undo manager")
 
 using namespace grt;
 using namespace base;
@@ -624,7 +620,6 @@ UndoManager::UndoManager(GRT *grt)
   _is_redoing= false;
   _undo_limit= 0;
   _blocks= 0;
-  g_static_rec_mutex_init(&_mutex);
 }
 
 
@@ -632,7 +627,6 @@ UndoManager::~UndoManager()
 {
   _changed_signal.disconnect_all_slots(); // prevent emission in reset()
   reset();
-  g_static_rec_mutex_free(&_mutex);
 }
 
 
@@ -656,13 +650,13 @@ void UndoManager::enable_logging_to(std::ostream *stream)
 
 void UndoManager::lock() const
 {
-  g_static_rec_mutex_lock(&_mutex);
+  _mutex.lock();
 }
 
 
 void UndoManager::unlock() const
 {
-  g_static_rec_mutex_unlock(&_mutex);
+  _mutex.unlock();
 }
 
 
@@ -817,6 +811,8 @@ UndoGroup *UndoManager::begin_undo_group(UndoGroup *group)
   if (!group)
     group= new UndoGroup();
 
+  log_debug3("begin undo group: %s\n", group->description().c_str());
+
   add_undo(group);
   
   return group;
@@ -892,8 +888,7 @@ bool UndoManager::end_undo_group(const std::string &description, bool trim)
       }
     }
      */
-    static const char* const default_log_domain = "Undo Manager";
-    log_debug("end undo group: %s\n", description.c_str());
+    log_debug3("end undo group: %s\n", description.c_str());
     return true;
   }
 }
