@@ -24,10 +24,13 @@
 
 #include "grtpp_util.h"
 #include "base/string_utilities.h"
+#include "base/log.h"
 
 #include <glib.h>
 #include <functional>
 
+DEFAULT_LOG_DOMAIN("grt")
+
 using namespace grt;
 
 #ifdef HAVE_CONFIG_H
@@ -226,8 +229,8 @@ ValueRef grt::get_value_by_path(const ValueRef &root, const std::string &path)
     }
   }
   
-  //if (!ok)
-    //throw grt::bad_item("Invalid path element '"+prefix+"' in path: "+path);
+  if (!ok)
+    log_error("Invalid path element '%s' in path: %s\n", prefix.c_str(), path.c_str());
   return value;
 }
 
