@@ -1,4 +1,4 @@
-# Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -16,7 +16,7 @@
 # 02110-1301  USA
 
 from __future__ import with_statement
-
+import platform
 import threading
 import random
 import Queue
@@ -27,9 +27,11 @@ import sys
 import time
 import os
 
+import mforms
+
 import paramiko
-from grt import log_error, log_warning, log_debug, log_debug2
-_this_file = os.path.basename(__file__)
+from workbench.log import log_warning, log_error, log_debug, log_debug2
+from wb_common import SSHFingerprintNewError
 
 SSH_PORT = 22
 REMOTE_PORT = 3306
@@ -44,9 +46,13 @@ else:
     class WarningPolicy(paramiko.MissingHostKeyPolicy):
         def missing_host_key(self, client, hostname, key):
             import binascii
-            log_warning(_this_file, 'WARNING: Unknown %s host key for %s: %s\n' % (key.get_name(), hostname, binascii.hexlify(key.get_fingerprint())))
+            log_warning('WARNING: Unknown %s host key for %s: %s\n' % (key.get_name(), hostname, binascii.hexlify(key.get_fingerprint())))
 
+class StoreIfConfirmedPolicy(paramiko.MissingHostKeyPolicy):
+    def missing_host_key(self, client, hostname, key):
+        raise SSHFingerprintNewError("Key mismatched", client, hostname, key)
 
+tunnel_serial=0
 class Tunnel(threading.Thread):
     
     """This class is a threaded implementation of an SSH tunnel.
@@ -86,16 +92,29 @@ class Tunnel(threading.Thread):
         self._shutdown = False
         self.connecting = False
         
-        self._client = None
+        self._client = paramiko.SSHClient()
         self._connections = []
-    
+
+
     def is_connecting(self):
         with self.lock:
             return self.connecting
-    
+
+
     def run(self):
+        try:
+            self.do_run()
+        except Exception, e:
+            log_error("Unhandled exception in SSH tunnel: %s\n" % e)
+
+
+    def do_run(self):
+        global tunnel_serial
+        tunnel_serial += 1
+        mforms.Utilities.set_thread_name("SSHTunnel%i"%tunnel_serial)
+
         sys.stdout.write('Thread started\n')  # sys.stdout.write is thread safe while print isn't
-        log_debug2(_this_file,"SSH Tunel thread started\n")
+        log_debug2("SSH Tunel %i thread started\n" % tunnel_serial)
         # Create a socket and pick a random port number for it:
         self._listen_sock = socket.socket()
         while True:
@@ -114,27 +133,29 @@ class Tunnel(threading.Thread):
                     self.notify_exception_error('ERROR',"Error initializing server end of tunnel", sys.exc_info())
                     raise exc
                 finally:
-                    self.port_is_set.set()
+                    with self.lock:
+                        self.connecting = True
 
-        with self.lock:
-            self.connecting = True
+                    self.port_is_set.set()
 
         if self._keyfile:
             self.notify('INFO', 'Connecting to SSH server at %s:%s using key %s...' % (self._server[0], self._server[1], self._keyfile) )
         else:
             self.notify('INFO', 'Connecting to SSH server at %s:%s...' % (self._server[0], self._server[1]) )
 
-        if not self._connect_ssh():
+        connected = self._connect_ssh()
+        if not connected:
             self._listen_sock.close()
             self._shutdown = True
-        else:
-            self.notify('INFO', 'Connection opened')
 
         with self.lock:
             self.connecting = False
+            
+        if connected:
+            self.notify('INFO', 'Connection opened')
 
         del self._password
-
+        last_activity = time.time()
         while not self._shutdown:
             try:
                 socks = [self._listen_sock]
@@ -147,12 +168,13 @@ class Tunnel(threading.Thread):
                     self.notify_exception_error('ERROR', 'Error while forwarding data: %r' % e, sys.exc_info())
                 break
 
-            if not r and len(socks) <= 1:
-                self.notify('INFO', 'Closing tunnel to %s:%s for inactivity...\n' % (self._server[0], self._server[1]) )
+            if not r and len(socks) <= 1 and time.time() - last_activity > TUNNEL_TIMEOUT:
+                self.notify('INFO', 'Closing tunnel to %s:%s for inactivity...' % (self._server[0], self._server[1]) )
                 break
+            last_activity = time.time()
 
             if self._listen_sock in r:
-                self.notify('INFO', 'New client connection\n')
+                self.notify('INFO', 'New client connection')
                 self.accept_client()
             
             closed = []
@@ -181,9 +203,12 @@ class Tunnel(threading.Thread):
                     chan.close()
                 except:
                     pass
-                self.notify('INFO', 'Client for %s disconnected\n' % local_port)
+                self.notify('INFO', 'Client for %s disconnected' % local_port)
                 self._connections.remove(item)
 
+            if closed and not self._connections and time.time() - last_activity > TUNNEL_TIMEOUT:
+                self.notify('INFO', 'Closing tunnel to %s:%s for inactivity...' % (self._server[0], self._server[1]) )
+                break
         # Time to shutdown:
         for sock, chan in self._connections:
             try:
@@ -197,14 +222,15 @@ class Tunnel(threading.Thread):
         
         self._listen_sock.close()
         self._client.close()
+        log_debug("Leaving tunnel thread %s\n" % self.local_port)
         
     def notify(self, msg_type, msg_object):
-        log_debug2(_this_file, "tunnel_%i: %s %s\n" % (self.local_port, msg_type, msg_object))
+        log_debug2("tunnel_%i: %s %s\n" % (self.local_port, msg_type, msg_object))
         self.q.put((msg_type, msg_object))
         
     def notify_exception_error(self, msg_type, msg_txt, msg_obj = None):
         self.notify(msg_type, msg_txt)
-        log_error(_this_file, traceback.format_exc())
+        log_error("%s\n" % traceback.format_exc())
 
     def match(self, server, username, target):
         with self.lock:
@@ -221,19 +247,27 @@ class Tunnel(threading.Thread):
                                                this is a subclass of paramiko.AuthenticationException
         """
         try:
+            self._client.get_host_keys().clear()
+            ssh_known_hosts_file = '~/.ssh/known_hosts'
             
-            self._client = paramiko.SSHClient()
-            self._client.load_system_host_keys()
-            self._client.set_missing_host_key_policy(WarningPolicy())
+            if platform.system().lower() == "windows":
+                ssh_known_hosts_file = '%s\ssh\known_hosts' % mforms.App.get().get_user_data_folder()
+
+            try:
+                self._client.load_host_keys(os.path.expanduser(ssh_known_hosts_file))
+            except IOError, e:
+                log_warning("IOError, probably caused by file %s not found, the message was: %s\n" % (ssh_known_hosts_file, e))
+
+            self._client.set_missing_host_key_policy(StoreIfConfirmedPolicy())
             has_key = bool(self._keyfile)
             self._client.connect(self._server[0], self._server[1], username=self._username,
                                  key_filename=self._keyfile, password=self._password,
                                  look_for_keys=has_key, allow_agent=has_key)
         except paramiko.BadHostKeyException, exc:
-            if sys.platform == "win32":
-                self.notify_exception_error('ERROR', "Delete entries for the host from the SSH known hosts file", sys.exc_info())
+            if platform.system().lower() == "windows":
+                self.notify_exception_error('ERROR', "%s\nDelete entries for the host from the %s file" % (str(exc), '%s\ssh\known_hosts' % mforms.App.get().get_user_data_folder()))
             else:
-                self.notify_exception_error('ERROR', "Delete entries for the host from the ~/.ssh/known_hosts file", sys.exc_info())
+                self.notify_exception_error('ERROR', "%s\nDelete entries for the host from the ~/.ssh/known_hosts file" % str(exc))
             return False
         except paramiko.BadAuthenticationType, exc:
             self.notify_exception_error('ERROR', "Bad authentication type, the server is not accepting this type of authentication.\nAllowed ones are:\n %s" % exc.allowed_types, sys.exc_info());
@@ -247,11 +281,19 @@ class Tunnel(threading.Thread):
         except paramiko.ChannelException, exc:
             self.notify_exception_error('ERROR', "Error connecting SSH channel.\nPlease refer to logs for details: %s" % str(exc), sys.exc_info())
             return False
+        except SSHFingerprintNewError, exc:
+            self.notify_exception_error('KEY_ERROR', { 'msg': "The authenticity of host '%(0)s (%(0)s)' can't be established.\nECDSA key fingerprint is %(1)s\nAre you sure you want to continue connecting?"  % {'0': "%s:%s" % (self._server[0], self._server[1]), '1': exc.fingerprint}, 'obj': exc})
+            return False
+        except IOError, exc:
+            #Io should be report to the user, so maybe he will be able to fix this issue
+            self.notify_exception_error('IO_ERROR', "IO Error: %s.\n Please refer to logs for details." % str(exc), sys.exc_info())
+            return False
+
         except Exception, exc:
-            print exc
             self.notify_exception_error('ERROR', "Authentication error, unhandled exception caught in tunnel manager, please refer to logs for details", sys.exc_info())
             return False
         else:
+            log_debug("connect_ssh2 OK\n")
             return True
 
     def close(self):
@@ -335,6 +377,9 @@ class TunnelManager:
         password = password or ''
         keyfile  = keyfile  or None
 
+        if keyfile is not None:
+            keyfile = keyfile.decode('utf-8')
+
         found = None
         for tunnel in self.tunnel_by_port.itervalues():
             if tunnel.match(server, username, target) and tunnel.isAlive():
@@ -360,6 +405,7 @@ class TunnelManager:
         if not tunnel:
             return 'Could not find a tunnel for port %d' % port
         error = None
+        tunnel.port_is_set.wait()
         if tunnel.isAlive():
             while True:
                 # Process any message in queue. Every retrieved message is printed.
@@ -367,13 +413,37 @@ class TunnelManager:
                 try:
                     msg_type, msg = tunnel.q.get_nowait()
                 except Queue.Empty:
-                    pass
+                    continue
                 else:
+                    if msg_type == 'KEY_ERROR':
+                        if mforms.Utilities.show_message("SSH Server Fingerprint Missing", msg['msg'], "Continue", "Cancel", "") == mforms.ResultOk:
+                            msg['obj'].client._host_keys.add(msg['obj'].hostname, msg['obj'].key.get_name(), msg['obj'].key)
+                            if msg['obj'].client._host_keys_filename is not None:
+                                try:
+                                    if os.path.isdir(os.path.dirname(msg['obj'].client._host_keys_filename)) == False:
+                                        log_warning("Host_keys directory is missing, recreating it\n")
+                                        os.makedirs(os.path.dirname(msg['obj'].client._host_keys_filename))
+                                    if os.path.exists(msg['obj'].client._host_keys_filename) == False:
+                                        log_warning("Host_keys file is missing, recreating it\n")
+                                        open(msg['obj'].client._host_keys_filename, 'a').close()
+                                    msg['obj'].client.save_host_keys(msg['obj'].client._host_keys_filename)
+                                    log_warning("Successfully saved host_keys file.\n")
+                                except IOError, e:
+                                    error = str(e)
+                                    break;
+                            error = "Server key has been stored"
+                        else:
+                            error = "User cancelled"
+                        break # Exit returning the error message
+                    elif msg_type == 'IO_ERROR':
+                        error = msg
+                        break # Exit returning the error message
+
                     _msg = msg
                     if type(msg) is tuple:
                         msg = '\n' + ''.join(traceback.format_exception(*msg))
                         _msg = str(_msg[1])
-                    log_debug(_this_file, "%s: %s\n" % (msg_type, msg))
+                    log_debug("%s: %s\n" % (msg_type, msg))
                     if msg_type == 'ERROR':
                         error = _msg
                         break  # Exit returning the error message
@@ -381,14 +451,20 @@ class TunnelManager:
                 if not tunnel.is_connecting() or not tunnel.isAlive():
                     break
                 time.sleep(0.3)
-        log_debug(_this_file, "returning from wait_connection(%s): %s\n" % (port, error))
+        log_debug("returning from wait_connection(%s): %s\n" % (port, error))
         return error
 
 
     def get_message(self, port):
+        if port not in self.tunnel_by_port:
+            log_error("Looking up invalid port %s\n" % port)
+            return None
         tunnel = self.tunnel_by_port[port]
-        return tunnel.q.get(block=False)
-        
+        try:
+            return tunnel.q.get_nowait()
+        except Queue.Empty:
+            return None
+
 
     def close(self, port):
         pass
@@ -457,7 +533,7 @@ class TunnelManager:
                 else:
                     self.send("NONE")
             else:
-                log_error(_this_file, "Invalid request %s\n" % request)
+                log_error("Invalid request %s\n" % request)
                 self.send("ERROR", "Invalid request")
 
 """
