@@ -1,4 +1,4 @@
-# Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -21,7 +21,8 @@ import string
 from wb import DefineModule
 import grt
 
-from db_generic_migration_grt import GenericMigration, find_object_with_name
+from db_generic_migration_grt import GenericMigration
+from workbench.utils import find_object_with_name
 
 ModuleInfo = DefineModule(name= "DbMssqlMigration", author= "Oracle Corp.", version="1.0")
 
@@ -206,17 +207,12 @@ class MSSQLMigration(GenericMigration):
 
             target_column.flags.extend(source_column.userType.flags)
 
-        # SQL expression to use for converting the column data to the target type
-        # eg.: CAST(? as NVARCHAR(max))
-        type_cast_expression = None
         if source_type:
             # Decide which mysql datatype corresponds to the column datatype:
             source_datatype = source_type.name.upper()
             # string data types:
             target_datatype = ''
             if source_datatype in ['VARCHAR', 'NVARCHAR']:
-                if source_datatype == 'VARCHAR':
-                    type_cast_expression = "CAST(? as NVARCHAR(%d))" % target_column.length
                 if source_column.length == -1:  # VARCHAR(MAX) or NVARCHAR(MAX)
                     target_datatype = 'LONGTEXT'  #TODO: Give the user the choice for this target datatype
                 elif 0 < source_column.length < 256:
@@ -224,12 +220,8 @@ class MSSQLMigration(GenericMigration):
                 else:  # MySQL versions > 5.0 can hold up to 65535 chars in a VARCHAR column
                     target_datatype = 'TEXT' if targetCatalog.version.majorNumber < 5 else 'VARCHAR'
             elif source_datatype in ['TEXT', 'NTEXT']:
-                if source_datatype == 'TEXT':
-                    type_cast_expression = "CAST(? as NTEXT)"
                 target_datatype = 'LONGTEXT'
             elif source_datatype in ['CHAR', 'NCHAR']:  # MSSQL CHAR's (also VARCHAR's) max length is 8000 non Unicode characters
-                if source_datatype == 'CHAR':
-                    type_cast_expression = "CAST(? as NCHAR(%d))" % target_column.length
                 if 0 < source_column.length < 256:
                     target_datatype = 'CHAR'
                 else:
@@ -245,7 +237,6 @@ class MSSQLMigration(GenericMigration):
                     target_column.flags.append('UNSIGNED')  # In MSSQL TINYINT is unsigned
             elif source_datatype == 'UNIQUEIDENTIFIER':
                 target_datatype = 'VARCHAR'
-                type_cast_expression = "CAST(? as VARCHAR(64))"
                 target_column.length = 64
                 if 'UNIQUE' not in target_column.flags:
                     target_column.flags.append('UNIQUE') # uniqueid must be UNIQUE... bug #43098
@@ -253,13 +244,22 @@ class MSSQLMigration(GenericMigration):
                         "Source column type %s was migrated to %s(%s)" % (source_datatype, target_datatype, target_column.length))
             elif source_datatype == 'SYSNAME':  # the relevant info is in http://msdn.microsoft.com/en-us/library/ms191240(v=sql.105).aspx
                 target_datatype = 'VARCHAR'
-                type_cast_expression = "CAST(? as VARCHAR(160))"
                 target_column.length = 160
                 state.addMigrationLogEntry(0, source_column, target_column,
                         "Source column type %s was migrated to %s(%s)" % (source_datatype, target_datatype, target_column.length))
             # floating point datatypes:
             elif source_datatype in ['DECIMAL', 'NUMERIC']:
-                target_datatype = 'DECIMAL'
+                if source_column.scale == 0:
+                    target_datatype = 'BIGINT'
+                    if source_column.precision < 5:
+                        target_datatype = 'SMALLINT'
+                    elif source_column.precision < 7:
+                        target_datatype = 'MEDIUMINT'
+                    elif source_column.precision < 10:
+                        target_datatype = 'INT'
+                    target_column.precision = -1
+                else:
+                    target_datatype = 'DECIMAL'
             elif source_datatype == 'REAL':
                 target_datatype = 'FLOAT'
             elif source_datatype == 'FLOAT':
@@ -288,8 +288,10 @@ class MSSQLMigration(GenericMigration):
                 target_datatype = 'DATETIME'
                 target_column.precision = -1
             # timestamp datatypes
+            # In MS SQL Server a nonnullable timestamp column is semantically equivalent to a binary(8) column, 
+            # and a nullable timestamp column is semantically equivalent to a varbinary(8) column.
             elif source_datatype in ['TIMESTAMP', 'ROWVERSION']:
-                target_datatype = 'TIMESTAMP'
+                target_datatype = 'BINARY' if source_column.isNotNull else 'VARBINARY'
             elif source_datatype == 'DATE':
                 target_datatype = 'DATE'
                 target_column.precision = -1
@@ -303,21 +305,17 @@ class MSSQLMigration(GenericMigration):
                       "Source column type BIT was migrated to TINYINT(1)")
             elif source_datatype == 'XML':
                 target_datatype = 'TEXT'
-                type_cast_expression = "CAST(? as NVARCHAR(max))"
                 state.addMigrationLogEntry(0, source_column, target_column,
                       "Source column type XML was migrated to TEXT")
             elif source_datatype in ['GEOMETRY', 'GEOGRAPHY']:
                 target_datatype = 'GEOMETRY'
-                type_cast_expression = '?.STAsText()'
             elif source_datatype == 'HIERARCHYID':
                 target_datatype = 'VARCHAR'
-                type_cast_expression = "CAST(? as VARCHAR(max))"
                 target_column.length = 255
                 state.addMigrationLogEntry(1, source_column, target_column,
                         "Source column type HIERARCHYID was migrated to VARCHAR(255)")
             elif source_datatype == 'SQL_VARIANT':
                 target_datatype = 'TEXT'
-                type_cast_expression = "CAST(? as NVARCHAR(max))"
                 state.addMigrationLogEntry(1, source_column, target_column,
                         "Source column type %s was migrated to %s(%s)" % (source_datatype, target_datatype, target_column.length))
             else:
@@ -331,9 +329,6 @@ class MSSQLMigration(GenericMigration):
                 state.addMigrationLogEntry(2, source_column, target_column, 
                     'Could not migrate column "%s" in "%s": Unknown datatype "%s"' % (target_column.name, source_column.owner.name, source_datatype) )
                 return False
-            
-            if type_cast_expression:
-                target_column.owner.customData["columnTypeCastExpression:%s" % target_column.name] = "%s as ?" % type_cast_expression
 
             return True
         else:
@@ -344,6 +339,59 @@ class MSSQLMigration(GenericMigration):
         return True
 
 
+    def migrateUpdateForChanges(self, state, target_catalog):
+        """
+        Create datatype cast expression for target column based on source datatype.
+        """
+        for targetSchema in target_catalog.schemata:
+            for targetTable in targetSchema.tables:
+                for target_column in targetTable.columns:
+                    # SQL expression to use for converting the column data to the target type
+                    # eg.: CAST(? as NVARCHAR(max))
+                    type_cast_expression = None
+                    source_datatype = None
+                    source_column = state.lookupSourceObject(target_column)
+                    if source_column:
+                        source_datatype = GenericMigration.getColumnDataType(self, source_column)
+                    if source_column and source_datatype:
+                        target_datatype = target_column.simpleType.name.upper()
+                        if source_datatype in ['VARCHAR', 'NVARCHAR']:
+                            if source_datatype == 'VARCHAR':
+                                if target_column.length > 4000 or target_column.length == -1: # NVARCHAR is limited to 4000 - if you need more, you must use MAX insted BUG #18167872
+                                    type_cast_expression = "CAST(? as NVARCHAR(MAX))"         # If in source table is VARCHAR(MAX) column lnegth is 0 - so must be casted to NVARCHAR(MAX) BUG #18105486
+                                else:
+                                    type_cast_expression = "CAST(? as NVARCHAR(%d))" % target_column.length
+                        elif source_datatype in ['TEXT', 'NTEXT']:
+                            if source_datatype == 'TEXT':
+                                type_cast_expression = "CAST(? as NTEXT)"
+                        elif source_datatype in ['CHAR', 'NCHAR']:  # MSSQL CHAR's (also VARCHAR's) max length is 8000 non Unicode characters
+                            if source_datatype == 'CHAR':
+                                type_cast_expression = "CAST(? as NCHAR(%d))" % target_column.length
+                        elif source_datatype == 'UNIQUEIDENTIFIER':
+                            type_cast_expression = "CAST(? as VARCHAR(64))"
+                        elif source_datatype == 'SYSNAME':  # the relevant info is in http://msdn.microsoft.com/en-us/library/ms191240(v=sql.105).aspx
+                            type_cast_expression = "CAST(? as VARCHAR(128))"
+                        # floating point datatypes:
+                        elif source_datatype in ['DECIMAL', 'NUMERIC']:
+                            if source_column.scale == 0:
+                                type_cast_expression = "CAST(? as %s)" % ('INT' if target_datatype == 'MEDIUMINT' else target_datatype)
+                        elif source_datatype == 'XML':
+                            type_cast_expression = "CAST(? as NVARCHAR(max))"
+                        elif source_datatype in ['GEOMETRY', 'GEOGRAPHY']:
+                            type_cast_expression = '?.STAsText()'
+                        elif source_datatype == 'HIERARCHYID':
+                            type_cast_expression = "CAST(? as VARCHAR(max))"
+                        elif source_datatype == 'SQL_VARIANT':
+                            type_cast_expression = "CAST(? as NVARCHAR(max))"
+                        elif source_datatype in ['TIMESTAMP', 'ROWVERSION']:
+                            type_cast_expression = 'CAST(? as %s)' % ('BINARY' if source_column.isNotNull else 'VARBINARY')
+
+                        if type_cast_expression:
+                            target_column.owner.customData["columnTypeCastExpression:%s" % target_column.name] = "%s as ?" % type_cast_expression
+
+        return target_catalog
+
+
     def migrateCharsetCollation(self, state, charset, collation, source_object, target_object):
         if collation:
             state.addMigrationLogEntry(0, source_object, target_object, 
@@ -427,6 +475,11 @@ def migrateRoutineToMySQL(state, source_routine, target_schema):
     return instance.migrateRoutineToMySQL(state, source_routine, target_schema)
 
 
+@ModuleInfo.export(grt.classes.db_Catalog, grt.classes.db_migration_Migration, grt.classes.db_Catalog)
+def migrateUpdateForChanges(state, target_catalog):
+    return instance.migrateUpdateForChanges(state, target_catalog)
+
+
 @ModuleInfo.export((grt.LIST, grt.classes.db_migration_MigrationParameter), grt.classes.db_migration_Migration)
 def getMigrationOptions(state):
     list = grt.List(grt.OBJECT, grt.classes.db_migration_MigrationParameter.__grtclassname__)
