@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -380,6 +380,8 @@ public:
   MySQLTriggerPanel(MySQLTableEditorBE *editor)
   : mforms::Box(true), _editor(editor), _list(mforms::TreeFlatList|mforms::TreeSizeSmall), _button(mforms::SmallButton), _refreshing(false)
   {
+    _edited_trigger_index = 0;
+
     set_spacing(8);
     set_padding(4);
     
@@ -397,9 +399,13 @@ public:
     _rtable.set_column_spacing(6);
     
     _name.set_name("trigger name");
-    _name.signal_changed()->connect(boost::bind(&MySQLTriggerPanel::name_changed, this));
+    _name.set_read_only(true);
+    //TODO: Check if we can make this code usable again
+//    _name.signal_changed()->connect(boost::bind(&MySQLTriggerPanel::name_changed, this));
     _definer.set_name("trigger definer");
-    _definer.signal_changed()->connect(boost::bind(&MySQLTriggerPanel::definer_changed, this));
+    _definer.set_read_only(true);
+    //TODO: Check if we can make this code usable again
+//    _definer.signal_changed()->connect(boost::bind(&MySQLTriggerPanel::definer_changed, this));
 
     _namel.set_style(mforms::SmallStyle);
     _definerl.set_style(mforms::SmallStyle);
@@ -463,7 +469,7 @@ public:
     {
       std::string t = (*trig)->timing();
       t.append(" ").append((*trig)->event());
-      trigmap.insert(std::make_pair(t, *trig));
+      trigmap.insert(std::make_pair(base::tolower(t), *trig));
     }
 //    std::sort(trigvec.begin(), trigvec.end(), &MySQLTriggerPanel::compare_order);
 
@@ -472,7 +478,7 @@ public:
     for (std::set<std::string>::const_iterator t = leftover.begin(); t != leftover.end(); ++t)
     {
       node = _list.add_node();
-      if ((it = trigmap.find(*t)) != trigmap.end())
+      if ((it = trigmap.find(base::tolower(*t))) != trigmap.end())
         node->set_string(0, it->second->name());
       else
         node->set_string(0, "-");
@@ -485,6 +491,29 @@ public:
     _refreshing = false;
   }
   
+  void reload_selected_trigger()
+  {
+    mforms::TreeNodeRef node = _list.get_selected_node();
+    if (!node)
+    {
+      _list.select_node(_list.node_at_row(0));
+      return;
+    }
+
+    std::string timing, event;
+    base::partition(node->get_string(1), " ", timing, event);
+    grt::ListRef<db_Trigger> triggers(_editor->get_table()->triggers());
+    GRTLIST_FOREACH(db_Trigger, triggers, trig)
+    {
+      if (base::string_compare((*trig)->timing(), timing, false) == 0 && base::string_compare((*trig)->event(), event, false) == 0)
+      {
+        _selected_trigger = *trig;
+        break;
+      }
+    }
+
+  }
+
   void code_edited()
   {
     // Don't run commit_changes() or the parser will kill your current trigger if it has an error.
@@ -493,43 +522,47 @@ public:
       if (_code_editor->is_dirty() && _selected_trigger->sqlDefinition() != _code_editor->get_string_value())
       {
         AutoUndoEdit undo(_editor, _selected_trigger, "sql");
-        grt::IntegerRef res = _editor->_sql_parser->parse_trigger(_selected_trigger, _code_editor->get_string_value().c_str());
+        _editor->freeze_refresh_on_object_change();
+        _editor->_sql_parser->parse_trigger(_selected_trigger, _code_editor->get_string_value().c_str());
+        _editor->thaw_refresh_on_object_change(true);
         _name.set_value(_selected_trigger->name());
         _definer.set_value(_selected_trigger->definer());
+        mforms::TreeNodeRef node = _list.node_at_row(_edited_trigger_index);
+        if (node)
+        {
+          node->set_string(0, _selected_trigger->name());
+        }
         undo.end(strfmt(_("Edit trigger `%s` of `%s`.`%s`"), _selected_trigger->name().c_str(), _editor->get_schema_name().c_str(), _editor->get_name().c_str()));
       }
     }
     else
-    {
-      _editor->commit_changes();
       refresh();
-    }
-  }
-  
-  void name_changed()
-  {
-    if (_selected_trigger.is_valid())
-    {
-      AutoUndoEdit undo(_editor, _selected_trigger, "name");
-      _selected_trigger->name(_name.get_string_value());
-      undo.end(base::strfmt("Rename trigger to %s", _name.get_string_value().c_str()));
-      mforms::TreeNodeRef node = _list.get_selected_node();
-      if (node)
-        node->set_string(0, _name.get_string_value());
-      selection_changed();
-    }
-  }
-
-  void definer_changed()
-  {
-    if (_selected_trigger.is_valid())
-    {
-      AutoUndoEdit undo(_editor, _selected_trigger, "definer");
-      _selected_trigger->definer(_definer.get_string_value());
-      undo.end(base::strfmt("Change trigger definer to %s", _definer.get_string_value().c_str()));
-      selection_changed();
-    }
   }
+  //TODO: Check if we can make this code usable again
+//  void name_changed()
+//  {
+//    if (_selected_trigger.is_valid())
+//    {
+//      AutoUndoEdit undo(_editor, _selected_trigger, "name");
+//      _selected_trigger->name(_name.get_string_value());
+//      undo.end(base::strfmt("Rename trigger to %s", _name.get_string_value().c_str()));
+//      mforms::TreeNodeRef node = _list.get_selected_node();
+//      if (node)
+//        node->set_string(0, _name.get_string_value());
+//      selection_changed();
+//    }
+//  }
+  //TODO: Check if we can make this code usable again
+//  void definer_changed()
+//  {
+//    if (_selected_trigger.is_valid())
+//    {
+//      AutoUndoEdit undo(_editor, _selected_trigger, "definer");
+//      _selected_trigger->definer(_definer.get_string_value());
+//      undo.end(base::strfmt("Change trigger definer to %s", _definer.get_string_value().c_str()));
+//      selection_changed();
+//    }
+//  }
 
   void clicked()
   {
@@ -577,7 +610,7 @@ public:
       update_editor();
     }
   }
-  
+
   void selection_changed()
   {
     if (_refreshing)
@@ -606,7 +639,7 @@ public:
     db_TriggerRef trigger;
     GRTLIST_FOREACH(db_Trigger, triggers, trig)
     {
-      if ((*trig)->timing() == timing && (*trig)->event() == event)
+      if (base::string_compare((*trig)->timing(), timing, false) == 0 && base::string_compare((*trig)->event(), event, false) == 0)
       {
         trigger = *trig;
         break;
@@ -639,6 +672,7 @@ public:
         else
           sql.append(trigger->sqlDefinition());
 
+        _edited_trigger_index = _list.row_for_node(_list.get_selected_node());
         _code_editor->set_text_keeping_state(sql.c_str());
       }
       else
@@ -652,10 +686,13 @@ public:
       _name.set_value("");
       _definer.set_value("");
       _name.set_enabled(false);
+      _name.set_read_only(true);
       _definer.set_enabled(false);
+      _definer.set_read_only(true);
       _button.set_text("Add Trigger");
       editor_enabled = false;
 
+      _edited_trigger_index = _list.row_for_node(_list.get_selected_node());
       sql = "-- Trigger not defined. Click Add Trigger to create it.\n";
       _code_editor->set_text_keeping_state(sql.c_str());
     }
@@ -675,6 +712,7 @@ private:
   mforms::Button _button;
   mforms::CodeEditor *_code_editor;
   db_TriggerRef _selected_trigger;
+  int _edited_trigger_index;
   bool _refreshing;
 };
 
@@ -718,10 +756,10 @@ std::vector<std::string> MySQLTableEditorBE::get_index_types()
   index_types.push_back("INDEX");
   index_types.push_back("UNIQUE");
   // FULLTEXT exists only in MyISAM prior to 5.6. in 5.6+ InnoDB also supports it
-  if (_table->tableEngine() == "MyISAM" || (_table->tableEngine() == "InnoDB" && is_server_version_at_least(5, 6)))
+  if (_table->tableEngine() == "MyISAM" || ((_table->tableEngine() == "InnoDB" || _table->tableEngine() == "") && is_server_version_at_least(5, 6)))
     index_types.push_back("FULLTEXT");
-  // SPATIAL is not supported by InnoDB
-  if (_table->tableEngine() == "MyISAM")
+  // SPATIAL is not supported by InnoDB before 5.7.5 (or maybe later)
+  if (_table->tableEngine() == "MyISAM" || ((_table->tableEngine() == "InnoDB" || _table->tableEngine() == "") && is_server_version_at_least(5, 7, 5)))
     index_types.push_back("SPATIAL");
 
   // these are special types for PK and FK
@@ -931,7 +969,7 @@ std::string MySQLTableEditorBE::get_table_option_by_name(const std::string& name
   return std::string("");
 }
 
-
+//TODO: Check if we still need this, if not ... remove.
 std::string MySQLTableEditorBE::get_all_triggers_sql() const
 {
   std::string retval;
@@ -998,16 +1036,16 @@ bool MySQLTableEditorBE::can_close()
 
 //--------------------------------------------------------------------------------------------------
 
+//TODO: Do we still need this?
 void MySQLTableEditorBE::commit_changes()
 {
-  mforms::CodeEditor* editor = get_sql_editor()->get_editor_control();
-  if (editor->is_dirty())
-  {
-    // TODO: no longer needed since we don't set all triggers at once anymore.
-    set_triggers_sql(get_all_triggers_sql(), true); //we need to always get all editors code not just the currently visible
-    // Note: don't reset the dirty state of the editor to be able to undo beyond this commit.
-  }
-
+//  mforms::CodeEditor* editor = get_sql_editor()->get_editor_control();
+//  if (editor->is_dirty())
+//  {
+//    // TODO: no longer needed since we don't set all triggers at once anymore.
+//    set_triggers_sql(get_all_triggers_sql(), true); //we need to always get all editors code not just the currently visible
+//    // Note: don't reset the dirty state of the editor to be able to undo beyond this commit.
+//  }
   // TODO: anything else to store?
 }
 
