@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2013 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2014 Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,6 +17,7 @@
  * 02110-1301  USA
  */
 
+#include "python_copy_data_source.h" // python stuff need to be 1st #include
 #include <cstdlib>
 #include <cstdio>
 #include <iostream>
@@ -29,11 +30,13 @@
 #include "boost/scoped_ptr.hpp"
 
 #include "copytable.h"
-#include "python_copy_data_source.h"
 #undef tolower
 #undef toupper
 
 #include "base/string_utilities.h"
+#include "base/file_utilities.h"
+
+#include "wb_version.h"
 
 #if defined(WIN32)
 #define atoll _atoi64
@@ -162,7 +165,7 @@ static void show_help()
   printf("<source schema><TAB><source table><TAB><target schema><TAB><target table><TAB>*|<select expression>\n");
   printf("Table Specification from command line:\n");
   printf("--table <source schema> <source table> <target schema> <target table> *|<select expression>\n");
-  printf("--table-range <source schema> <source table> <target schema> <target table> <source key> <start>|-1 <end>|-1\n");
+  printf("--table-range <source schema> <source table> <target schema> <target table> <select expression> <source key> <start>|-1 <end>|-1\n");
   printf("\n");
   printf("--log-file=<file_path>\n");
   printf("--log-level=<level>\n");
@@ -171,6 +174,7 @@ static void show_help()
   printf("--disable-triggers-on=<schema>\n");
   printf("--reenable-triggers-on=<schema>\n");
   printf("--dont-disable-triggers");
+  printf("--version\n");
   printf("--help\n");
 }
 
@@ -238,15 +242,16 @@ bool read_tasks_from_file(const std::string file_name, bool count_only, TaskQueu
 
 int main(int argc, char **argv)
 {
-  if (!g_thread_supported())
-    g_thread_init(NULL);
+  std::string app_name = base::basename(argv[0]);
 
+  base::threading_init();
+  
   TaskQueue tables;
 
   std::string source_password;
   std::string source_connstring;
   bool source_is_utf8 = false;
-  SourceType source_type;
+  SourceType source_type = ST_MYSQL;
 
   std::string target_connstring;
   std::string target_password;
@@ -351,6 +356,24 @@ int main(int argc, char **argv)
       if (bulk_insert_batch < 1)
         bulk_insert_batch = 100;
     }
+    else if (strcmp(argv[i], "--version") == 0)
+    {
+      const char *type = APP_EDITION_NAME;
+      if (strcmp(APP_EDITION_NAME, "Community") == 0)
+        type = "CE";
+
+      printf("%s %s (%s) %i.%i.%i %i %s build %i\n"
+             , base::basename(argv[0]).c_str()
+             , type, APP_LICENSE_TYPE
+             , APP_MAJOR_NUMBER
+             , APP_MINOR_NUMBER
+             , APP_RELEASE_NUMBER
+             , APP_REVISION_NUMBER
+             , APP_RELEASE_TYPE
+             , APP_BUILD_NUMBER
+            );
+      exit(0);
+    }
     else if (strcmp(argv[i], "--help") == 0 || strcmp(argv[i], "-h") == 0)
     {
       show_help();
@@ -389,7 +412,7 @@ int main(int argc, char **argv)
     {
       TableParam param;
 
-      if ((!count_only && i + 7 >= argc) || (count_only && i + 5 >= argc))
+      if ((!count_only && i + 8 >= argc) || (count_only && i + 5 >= argc))
       {
         fprintf(stderr, "%s: Missing value for table copy specification\n", argv[0]);
         exit(1);
@@ -400,6 +423,7 @@ int main(int argc, char **argv)
       {
         param.target_schema = argv[++i];
         param.target_table = argv[++i];
+        param.select_expression = argv[++i];
 
         trigger_schemas.insert(param.target_schema);
       }
@@ -600,7 +624,7 @@ int main(int argc, char **argv)
     else if (reenable_triggers || disable_triggers)
     {
       boost::scoped_ptr<MySQLCopyDataTarget> ptarget;
-      ptarget.reset(new MySQLCopyDataTarget(target_host, target_port, target_user, target_password, target_socket));
+      ptarget.reset(new MySQLCopyDataTarget(target_host, target_port, target_user, target_password, target_socket, app_name));
 
       if (disable_triggers)
         ptarget->backup_triggers(trigger_schemas);
@@ -617,7 +641,7 @@ int main(int argc, char **argv)
 
       if (disable_triggers_on_copy)
       {
-        ptarget_conn.reset(new MySQLCopyDataTarget(target_host, target_port, target_user, target_password, target_socket));
+        ptarget_conn.reset(new MySQLCopyDataTarget(target_host, target_port, target_user, target_password, target_socket, app_name));
         ptarget_conn->backup_triggers(trigger_schemas);
       }
 
@@ -635,7 +659,7 @@ int main(int argc, char **argv)
         else
           psource = new PythonCopyDataSource(source_connstring, source_password);
 
-        ptarget = new MySQLCopyDataTarget(target_host, target_port, target_user, target_password, target_socket);
+        ptarget = new MySQLCopyDataTarget(target_host, target_port, target_user, target_password, target_socket, app_name);
 
         psource->set_max_blob_chunk_size(ptarget->get_max_allowed_packet());
         psource->set_max_parameter_size(ptarget->get_max_long_data_size());
