@@ -1,4 +1,4 @@
-# Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -31,14 +31,13 @@ import wb_admin_control
 from wb_server_control import PasswordHandler
 from wb_server_control import ServerProfile
 from workbench.db_utils import MySQLConnection, MySQLError, escape_sql_string
-from wb_common import OperationCancelledError, InvalidPasswordError, NoDriverInConnection, Users
-from wb_server_management import local_get_cmd_output
+from wb_common import OperationCancelledError, InvalidPasswordError, NoDriverInConnection
 
+from workbench.utils import Version
 from workbench.notifications import NotificationCenter
 
 
-from grt import log_info, log_error, log_warning, log_debug
-_this_file = os.path.basename(__file__)
+from workbench.log import log_info, log_warning, log_error, log_debug
 
 
 # How the Administrator Module Works
@@ -107,7 +106,7 @@ class AdministratorContext:
 
         self.sidebar.add_on_section_command_callback(self._sidebar_entry_clicked)
 
-        self.sidebar_sections = [("wba_management", "MANAGEMENT", []), ("wba_instance", "INSTANCE", [])]
+        self.sidebar_sections = [("wba_management", "MANAGEMENT", []), ("wba_instance", "INSTANCE", []),  ("wba_performance", "PERFORMANCE", [])]
 
         self.shown_in_sidebar = False
 
@@ -133,7 +132,7 @@ class AdministratorContext:
 
     def handle_close(self):
         grt.root.wb.options.options['Administrator:sidebar_collapsed_sections'] = self.sidebar.get_collapse_states()
-        del self.editor.customData["adminContext"]
+        #del self.editor.customData["adminContext"]
 
         self.admin_tab = None
         self.ctrl_be = None
@@ -162,22 +161,6 @@ class AdministratorContext:
               # Retry
               self.server_profile = ServerProfile(self.connection, self.instance_profile, False)
 
-
-    def _check_server_version(self):
-        version = self.ctrl_be.get_server_version()
-        if type(version) is tuple:
-            valid_versions = ((4,0), (4,1), (5,0), (5,1), (5,2), (5,4), (5,5), (5,6), (5, 7))
-            if version[:2] not in valid_versions:
-                log_warning(_this_file, "%s: Server version %s is NOT supported\n" % (self.__class__.__name__, str(version)) )
-                Utilities.show_error("Unsupported Server Version", "The version of the server you're trying to connect to is %i.%i, which is not supported by Workbench."%version[:2],
-                                     "Close", "Ignore", "")
-                return False
-            else:
-                log_info(_this_file, "%s: Server version %s is supported\n" % (self.__class__.__name__, str(version)) )
-                return True
-        return None
-
-
     def _acquire_admin_access(self):
         if not self._validate_remote_admin_settings():
             self.admin_access_status = "Remote management settings are invalid"
@@ -268,7 +251,7 @@ class AdministratorContext:
     def _dock_admin_tab(self):
         app = mforms.App.get()
         try:
-            self.ctrl_be = wb_admin_control.WbAdminControl(self.server_profile, connect_sql=True)
+            self.ctrl_be = wb_admin_control.WbAdminControl(self.server_profile, self.editor, connect_sql=True)
             self.ctrl_be.init()
 
             self.admin_tab = wb_admin_main.AdministratorTab(self.ctrl_be, self.server_profile, self, self.editor)
@@ -291,16 +274,13 @@ class AdministratorContext:
             app.set_status_text("Could not Open WB Admin")
             return None
 
-        if self._check_server_version() is False:
-            app.set_status_text("Unsupported server version for Administration")
-            return None
-
-
         dp = mforms.fromgrt(self.editor.dockingPoint)
         dp.dock_view(self.admin_tab, "", 0)
         dp.select_view(self.admin_tab)
         self.admin_tab.set_title("Administrator")
 
+        return self.admin_tab
+
 
     def become_active_tab(self):
         dp = mforms.fromgrt(self.editor.dockingPoint)
@@ -321,7 +301,8 @@ class AdministratorContext:
 
         # if this is the 1st time opening the WBA, init the main tab and dock it
         if self.admin_tab is None:
-            self._dock_admin_tab()
+            if not self._dock_admin_tab():
+                return
 
         page = self.page_instances.get(entry_id)
         if not page:
@@ -360,15 +341,28 @@ class AdministratorContext:
 
     def show_in_sidebar(self):
         if not self.shown_in_sidebar:
+            if self.editor.serverVersion:
+                server_version = Version.fromgrt(self.editor.serverVersion)
+            else:
+                server_version = None
             self.shown_in_sidebar = True
+            first = True
             for sname, stitle, sitems in self.sidebar_sections:
-                self.sidebar.add_section(sname, stitle, mforms.TaskSectionShowConfigButton if sname == "wba_instance" else mforms.TaskSectionPlain)
+                flags = mforms.TaskSectionShowConfigButton if sname == "wba_instance" else mforms.TaskSectionPlain
+                if first:
+                    flags |= mforms.TaskSectionToggleModeButton
+                    first = False
+                self.sidebar.add_section(sname, stitle, flags)
                 for ident, ititle, icon_path in sitems:
                     self.sidebar.add_section_entry(sname, ident, ititle, icon_path, mforms.TaskEntryLink)
-                    requires_remote_access = self.admin_pages.get(ident, (None, True))[1]
+                    mod, requires_remote_access = self.admin_pages.get(ident, (None, True))
                     enabled = True
                     if requires_remote_access and (not self.server_profile or (self.server_profile and not self.server_profile.is_local and not self.server_profile.remote_admin_enabled)):
                         enabled = False
+                    elif getattr(mod, "min_server_version", None):
+                        if server_version and not server_version.is_supported_mysql_version_at_least(*mod.min_server_version):
+                            enabled = False
+
                     self.sidebar.set_section_entry_enabled(ident, enabled)
 
             self.sidebar.set_collapse_states(grt.root.wb.options.options.get('Administrator:sidebar_collapsed_sections', ''))
@@ -385,7 +379,8 @@ def attachToSQLEditor(name, sender, args):
     context = AdministratorContext(sender)
     sender.customData["adminContext"] = grt.togrt(context)
     if not sender.isConnected:
-        mforms.Utilities.add_timeout(0.1, lambda:context.open_into_section("admin_server_status", True))
+        ignore = mforms.Utilities.add_timeout(0.1, lambda:context.open_into_section("admin_server_status", True))
+        del ignore
 
 #-------------------------------------------------------------------------------
 def handleReconnect(name, sender, args):
@@ -409,7 +404,6 @@ def initialize():
 wba_page_modules = wb_admin_main.scan_admin_modules()
 
 #-------------------------------------------------------------------------------
-
 @ModuleInfo.export(grt.classes.db_mgmt_ServerInstance, grt.classes.db_mgmt_Connection)
 def autoDetectLocalInstance(connection):
     """Create a Instance profile for the local server from the connection."""
@@ -422,27 +416,35 @@ def autoDetectLocalInstance(connection):
     if version:
         version = ".".join(version.split(".")[:2])
 
-    def get_profiles_for(system):
+    def get_profiles_for(system, version):
         path = mforms.App.get().get_resource_path("mysql.profiles")
         if not path:
             path = mforms.App.get().get_resource_path("")
             if not path:
-                log_error(_this_file, "Could not find mysql.profiles dir\n")
+                log_error("Could not find mysql.profiles dir\n")
                 return []
             path += "/mysql.profiles"
+        version = Version.fromstr(version or "5.6")
         files = os.listdir(path)
         profiles = []
+        matched_profiles = []
         for f in files:
             data = grt.unserialize(os.path.join(path, f))
             if data.has_key("sys.system") and data["sys.system"] == system:
                 profiles.append(data)
+                profile_version = Version.fromstr(data.get("serverVersion"))
+                if version.majorNumber == profile_version.majorNumber or version.minorNumber == profile_version.minorNumber:
+                    matched_profiles.append(data)
+
+        if matched_profiles:
+            return matched_profiles
         return profiles
 
     def pick_suitable_linux_profile(profiles):
         return profiles[0]
 
     if sys.platform.lower().startswith("win"):
-        profiles = get_profiles_for("Windows")
+        profiles = get_profiles_for("Windows", version)
         if profiles:
             if version:
                 profiles = [prof for prof in profiles if prof.get("serverVersion", None) == version]
@@ -450,7 +452,7 @@ def autoDetectLocalInstance(connection):
                 instance.serverInfo.update(profiles[0])
         instance.serverInfo["windowsAdmin"] = 1 # this forces WMI admin for localhost Windows
     elif sys.platform.lower() == "darwin":
-        profiles = get_profiles_for("MacOS X")
+        profiles = get_profiles_for("MacOS X", version)
         if profiles:
             instance.serverInfo.update(profiles[0])
             possible_paths = [instance.serverInfo.get("sys.config.path"), "/etc/my.cnf", "/etc/mysql/my.cnf"]
@@ -459,7 +461,7 @@ def autoDetectLocalInstance(connection):
                     instance.serverInfo["sys.config.path"] = p
                     break
     elif "linux" in sys.platform.lower():
-        profiles = get_profiles_for("Linux")
+        profiles = get_profiles_for("Linux", version)
         if profiles:
             profile = pick_suitable_linux_profile(profiles)
             instance.serverInfo.update(profile)
@@ -606,15 +608,15 @@ class PasswordExpiredDialog(mforms.Form):
                 if mforms.Utilities.show_error("Reset Password", str(e), "Retry", "Cancel", "") == mforms.ResultOk:
                     retry = True
                 result = 0
-          
-            if old_script is not None:
-                con.parameterValues["preInit"] = old_script
-            else:
-                del con.parameterValues["preInit"]
-            if old_multi_statements is not None:
-                con.parameterValues["CLIENT_MULTI_STATEMENTS"] = old_multi_statements
-            else:
-                del con.parameterValues["CLIENT_MULTI_STATEMENTS"]
+            finally:
+                if old_script is not None:
+                    con.parameterValues["preInit"] = old_script
+                else:
+                    del con.parameterValues["preInit"]
+                if old_multi_statements is not None:
+                    con.parameterValues["CLIENT_MULTI_STATEMENTS"] = old_multi_statements
+                else:
+                    del con.parameterValues["CLIENT_MULTI_STATEMENTS"]
 
             if retry:
                 return self.run()
@@ -653,7 +655,7 @@ test_ssh_connection_is_windows = None
 def testInstanceSettingByName(what, connection, server_instance):
     global test_ssh_connection
 
-    log_debug(_this_file, "Test %s in %s\n" % (what, connection.name))
+    log_debug("Test %s in %s\n" % (what, connection.name))
 
     profile = ServerProfile(connection, server_instance)
 
@@ -661,10 +663,10 @@ def testInstanceSettingByName(what, connection, server_instance):
         if test_ssh_connection:
             test_ssh_connection = None
 
-        log_info(_this_file, "Instance test: Connecting to %s\n" % profile.ssh_hostname)
+        log_info("Instance test: Connecting to %s\n" % profile.ssh_hostname)
 
         try:
-            test_ssh_connection = wb_admin_control.WbAdminControl(profile, connect_sql=False)
+            test_ssh_connection = wb_admin_control.WbAdminControl(profile, None, connect_sql=False)
             test_ssh_connection.init()
             grt.send_info("connected.")
         except Exception, exc:
@@ -685,13 +687,13 @@ def testInstanceSettingByName(what, connection, server_instance):
         os_info = test_ssh_connection.detect_operating_system_version()
         if os_info:
             os_type, os_name, os_variant, os_version = os_info
-            log_info(_this_file, "Instance test: detected remote OS: %s (%s), %s, %s\n" % (os_info))
+            log_info("Instance test: detected remote OS: %s (%s), %s, %s\n" % (os_info))
 
             # check if the admin access error was because of wrong OS set
             if os_type != profile.target_os:
                 return "ERROR Wrong Remote OS configured for connection. Set to %s, but was detected as %s" % (profile.target_os, os_type)
         else:
-            log_warning(_this_file, "Instance test: could not determine OS version information\n")
+            log_warning("Instance test: could not determine OS version information\n")
 
             return "ERROR Could not determine remote OS details"
 
@@ -736,7 +738,7 @@ def testInstanceSettingByName(what, connection, server_instance):
 
     elif what in ("find_config_file/local", "check_config_path/local", "check_config_section/local"):
         config_file = profile.config_file_path
-        config_file = wb_admin_control.WbAdminControl(profile, connect_sql=False).expand_path_variables(config_file)
+        config_file = wb_admin_control.WbAdminControl(profile, None, connect_sql=False).expand_path_variables(config_file)
         print "Check if %s can be accessed" % config_file
         if os.path.exists(config_file):
             print "File was found at the expected location"
@@ -770,12 +772,6 @@ def testInstanceSettingByName(what, connection, server_instance):
             if cmd != cmd_start and not test_ssh_connection.ssh.file_exists(cmd):
                 return "ERROR %s is invalid" % path
 
-        command = profile.check_server_status_cmd
-        print "Checking command '%s'" % command
-        #rc = test_ssh_connection.is_running()
-        rc = test_ssh_connection.server_control.get_status()
-        print "Server detected as %s" % (rc and "running" or "stopped"),
-
         return "OK"
 
     elif what == "check_admin_commands/local":
@@ -792,12 +788,7 @@ def testInstanceSettingByName(what, connection, server_instance):
             if cmd != cmd_start and not os.path.exists(cmd):
                 return "ERROR %s is invalid" % path
 
-        command = profile.check_server_status_cmd
-        print "Checking command '%s'" % command
-        result, rc = local_get_cmd_output(command, Users.CURRENT, None)
-        print "Server detected as %s" % (result and "running" or "stopped")
-
-        return "OK "+(result and "running" or "stopped")
+        return "OK"
 
     return "ERROR bad command"
 
@@ -827,6 +818,8 @@ def openAdminSection(editor, section):
     context = grt.fromgrt(editor.customData["adminContext"])
     if context:
         context.open_into_section(section, True)
+    else:
+        log_error("No context found for editor in call to openAdminSection\n")
 
 
 # Hack to make this plugin only appear if SE modules are available
@@ -838,6 +831,8 @@ try:
         context = grt.fromgrt(editor.customData["adminContext"])
         if context:
             context.open_into_section(section, True)
+        else:
+            log_error("No context found for editor in call to openAdminSection\n")
 except ImportError:
     pass
 
