@@ -1,4 +1,4 @@
-# Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -24,18 +24,19 @@ import grt
 # import the mforms module for GUI stuff
 import mforms
 
-import re
 import os
 
+from workbench.log import log_error
 from workbench.notifications import NotificationCenter
 
-from sql_reformatter import formatter_for_statement_ast, ASTHelper
+from sql_reformatter import formatter_for_statement_ast
 from text_output import TextOutputTab
 from sqlide_catalogman_ext import show_schema_manager
+from sqlide_tableman_ext import show_table_inspector
 
 from sqlide_resultset_ext import handleResultsetContextMenu
 import sqlide_catalogman_ext
-#import sqlide_tableman_ext
+import sqlide_tableman_ext
 import sqlide_schematree_ext
 
 # define this Python module as a GRT module
@@ -52,7 +53,7 @@ def initialize0():
     # register a handler for when the SQLIDE live schema tree context menu is about to be shown
     nc.add_observer(sqlide_schematree_ext.handleLiveTreeContextMenu, name = "GRNLiveDBObjectMenuWillShow") # must be 1st
     nc.add_observer(sqlide_catalogman_ext.handleLiveTreeContextMenu, name = "GRNLiveDBObjectMenuWillShow")
-    #nc.add_observer(sqlide_tableman_ext.handleLiveTreeContextMenu, name = "GRNLiveDBObjectMenuWillShow")
+    nc.add_observer(sqlide_tableman_ext.handleLiveTreeContextMenu, name = "GRNLiveDBObjectMenuWillShow")
 
 
 
@@ -108,67 +109,70 @@ def importRecordsetDataFromFile(resultset):
 @ModuleInfo.plugin("wb.sqlide.executeToTextOutput", caption= "Execute Query Into Text Output", input= [wbinputs.currentQueryEditor()])
 @ModuleInfo.export(grt.INT, grt.classes.db_query_QueryEditor)
 def executeQueryAsText(qbuffer):
-    view = TextOutputTab("")
-
-    dock = mforms.fromgrt(qbuffer.resultDockingPoint)
-    dock.dock_view(view, "", 0)
-    dock.select_view(view)
-    dock.set_view_title(view, "Query Output")
-
+  
     editor = qbuffer.owner
     sql = qbuffer.selectedText or qbuffer.script
     resultsets = editor.executeScript(sql)
-    for result in resultsets:
-        output = ["Execute:"]
-        output.append("> %s\n" % result.sql)
-        line = []
-        column_lengths = []
-        ncolumns = len(result.columns)
-        for column in result.columns:
-            line.append(column.name + " "*5)
-            column_lengths.append(len(column.name)+5)
+  
+    if resultsets:
+        view = TextOutputTab("")
 
-        separator = []
-        for c in column_lengths:
-            separator.append("-"*c)
-        separator = " + ".join(separator)
-        output.append("+ "+separator+" +")
+        dock = mforms.fromgrt(qbuffer.resultDockingPoint)
+        dock.dock_view(view, "", 0)
+        view.set_title("Query Output")
+        dock.select_view(view)
 
-        line = " | ".join(line)
-        output.append("| "+line+" |")
-
-        output.append("+ "+separator+" +\n")
-
-        ok = result.goToFirstRow()
-        if ok:
-            view.textbox.append_text('\n'.join(output))
-
-        import time
-        
-        last_flush = 0
-        rows = []
-        while ok:
+        for result in resultsets:
+            output = ["Execute:"]
+            output.append("> %s\n" % result.sql)
             line = []
-            for i in range(ncolumns):
-              value = result.stringFieldValue(i)
-              if value is None:
-                value = "NULL"
-              line.append(value.ljust(column_lengths[i]))
-            line= " | ".join(line)
-            rows.append("| "+line+" |")
+            column_lengths = []
+            ncolumns = len(result.columns)
+            for column in result.columns:
+                line.append(column.name + " "*5)
+                column_lengths.append(len(column.name)+5)
+
+            separator = []
+            for c in column_lengths:
+                separator.append("-"*c)
+            separator = " + ".join(separator)
+            output.append("+ "+separator+" +")
+
+            line = " | ".join(line)
+            output.append("| "+line+" |")
+
+            output.append("+ "+separator+" +\n")
+
+            ok = result.goToFirstRow()
+            if ok:
+                view.textbox.append_text('\n'.join(output))
+
+            import time
             
-            # flush text every 1/2s
-            if time.time() - last_flush >= 0.5:
-                last_flush = time.time()
+            last_flush = 0
+            rows = []
+            while ok:
+                line = []
+                for i in range(ncolumns):
+                  value = result.stringFieldValue(i)
+                  if value is None:
+                    value = "NULL"
+                  line.append(value.ljust(column_lengths[i]))
+                line= " | ".join(line)
+                rows.append("| "+line+" |")
+                
+                # flush text every 1/2s
+                if time.time() - last_flush >= 0.5:
+                    last_flush = time.time()
+                    view.textbox.append_text("\n".join(rows)+"\n")
+                    rows = []
+                ok = result.nextRow()
+
+            if rows:
                 view.textbox.append_text("\n".join(rows)+"\n")
-                rows = []
-            ok = result.nextRow()
 
-        if rows:
-            view.textbox.append_text("\n".join(rows)+"\n")
-
-        view.textbox.append_text("+ "+separator+" +\n")
-        view.textbox.append_text("%i rows\n\n" % (result.currentRow + 1))
+            view.textbox.append_text("+ "+separator+" +\n")
+            view.textbox.append_text("%i rows\n\n" % (result.currentRow + 1))
 
     return 0
 
@@ -212,10 +216,6 @@ def doReformatSQLStatement(text, return_none_if_unsupported):
     if type(ast_list[0]) is str:
         raise Exception("Error parsing statement: %s" % ast_list[0])
 
-    helper = ASTHelper(text)
-
-    curpos = 0
-    new_text = ""
     ast = ast_list[0]
     
     def trim_ast_fix_bq(text, node):
@@ -251,13 +251,15 @@ def reformatSQLStatement(text):
 @ModuleInfo.plugin("wb.sqlide.enbeautificate", caption = "Reformat SQL Query", input=[wbinputs.currentQueryBuffer()])
 @ModuleInfo.export(grt.INT, grt.classes.db_query_QueryBuffer)
 def enbeautificate(editor):
+    """Reformat the selected SQL statements or the one under the cursor."""
+
     from grt.modules import MysqlSqlFacade
 
     text = editor.selectedText
     selectionOnly = True
     if not text:
         selectionOnly = False
-        text = editor.script
+        text = editor.currentStatement
 
     ok_count = 0
     bad_count = 0
@@ -327,7 +329,7 @@ def enbeautificate(editor):
     if selectionOnly:
         editor.replaceSelection(new_text)
     else:
-        editor.replaceContents(new_text)
+        editor.replaceCurrentStatement(new_text)
 
     if bad_count > 0:
         mforms.App.get().set_status_text("Formatted %i statements, %i unsupported statement types skipped."%(ok_count, bad_count))
@@ -567,13 +569,29 @@ def commentText(editor):
         editor.insertionPoint = pos
     return 0
 
-
 @ModuleInfo.export(grt.INT, grt.classes.db_query_Editor, grt.LIST)
-def showSchemaInspector(editor, selection):
-    show_schema_manager(editor, selection, False)
+def showInspector(editor, selection):
+    schema_insp = []
+    table_insp = []
+    table_insp_idx = []
+    for s in selection:
+        if s.type == "db.Schema":
+            schema_insp.append(s.schemaName)
+        elif s.type == "db.Table":
+            table_insp.append((s.schemaName, s.name))
+        elif s.type == "db.Index":
+            table_insp_idx.append((s.schemaName, s.owner.name))
+        else:
+            log_error("Unsupported inspector type: %s\n" % s.type)
+            
+    if len(schema_insp):
+        show_schema_manager(editor, schema_insp, False)
+    if len(table_insp):
+        show_table_inspector(editor, table_insp)
+    if len(table_insp_idx):
+        show_table_inspector(editor, table_insp_idx, "indexes")
     return 0
 
-
 #@ModuleInfo.plugin("wb.sqlide.refactor.renameSchema", caption = "Rename References to Schema Name", input=[wbinputs.currentQueryBuffer()]) # pluginMenu="SQL/Utilities"
 #@ModuleInfo.export(grt.INT, grt.classes.db_query_QueryBuffer)
   #def refactorRenameSchema(editor):
